@@ -1,19 +1,19 @@
 ﻿// Licensed to the .NET Foundation under one or more agreements.
 // The .NET Foundation licenses this file to you under the MIT license.
-// See the LICENSE file in the project root for more information.
 
-using Microsoft.Diagnostics.DebugServices;
-using Microsoft.Diagnostics.DebugServices.Implementation;
-using Microsoft.Diagnostics.ExtensionCommands;
-using Microsoft.Diagnostics.Repl;
-using Microsoft.Diagnostics.Runtime;
-using Microsoft.Diagnostics.Runtime.Utilities;
-using SOS.Hosting;
 using System;
 using System.Collections.Generic;
 using System.Diagnostics;
 using System.Reflection;
 using System.Runtime.InteropServices;
+using System.Text;
+using Microsoft.Diagnostics.DebugServices;
+using Microsoft.Diagnostics.DebugServices.Implementation;
+using Microsoft.Diagnostics.ExtensionCommands;
+using Microsoft.Diagnostics.Runtime;
+using Microsoft.Diagnostics.Runtime.Utilities;
+using SOS.Hosting;
+using SOS.Hosting.DbgEng.Interop;
 
 namespace SOS.Extensions
 {
@@ -22,7 +22,7 @@ namespace SOS.Extensions
     /// </summary>
     public sealed unsafe class HostServices : COMCallableIUnknown, IHost
     {
-        private static readonly Guid IID_IHostServices = new Guid("27B2CB8D-BDEE-4CBD-B6EF-75880D76D46F");
+        private static readonly Guid IID_IHostServices = new("27B2CB8D-BDEE-4CBD-B6EF-75880D76D46F");
 
         /// <summary>
         /// This is the prototype of the native callback function.
@@ -30,32 +30,32 @@ namespace SOS.Extensions
         /// <param name="hostServices">The instance of the host services for the native code to use</param>
         /// <returns></returns>
         [UnmanagedFunctionPointer(CallingConvention.Cdecl)]
-        private delegate HResult InitializeCallbackDelegate(
+        private delegate int InitializeCallbackDelegate(
             IntPtr hostServices);
 
         internal IntPtr IHostServices { get; }
 
         internal DebuggerServices DebuggerServices { get; private set; }
 
-        private readonly ServiceProvider _serviceProvider;
-        private readonly CommandProcessor _commandProcessor;
+        private readonly ServiceManager _serviceManager;
+        private readonly CommandService _commandService;
         private readonly SymbolService _symbolService;
         private readonly HostWrapper _hostWrapper;
+        private ServiceContainer _serviceContainer;
         private ContextServiceFromDebuggerServices _contextService;
         private int _targetIdFactory;
         private ITarget _target;
-        private TargetWrapper _targetWrapper;
-        private IMemoryService _memoryService;
 
         /// <summary>
         /// Enable the assembly resolver to get the right versions in the same directory as this assembly.
         /// </summary>
         static HostServices()
         {
-            if (RuntimeInformation.FrameworkDescription.StartsWith(".NET Framework")) {
+            if (RuntimeInformation.FrameworkDescription.StartsWith(".NET Framework"))
+            {
                 AssemblyResolver.Enable();
             }
-            LoggingCommand.Initialize();
+            DiagnosticLoggingService.Initialize();
         }
 
         /// <summary>
@@ -63,6 +63,16 @@ namespace SOS.Extensions
         /// </summary>
         public static HostServices Instance { get; private set; }
 
+        /// <summary>
+        /// The time out in minutes passed to the HTTP symbol store
+        /// </summary>
+        public static int DefaultTimeout { get; set; } = 4;
+
+        /// <summary>
+        /// The retry count passed to the HTTP symbol store
+        /// </summary>
+        public static int DefaultRetryCount { get; set; }
+
         /// <summary>
         /// This is the main managed entry point that the native hosting code calls. It needs to be a single function
         /// and is restricted to just a string parameter because host APIs (i.e. desktop clr) have this narrow interface.
@@ -77,7 +87,7 @@ namespace SOS.Extensions
             {
                 extensionLibrary = DataTarget.PlatformFunctions.LoadLibrary(extensionPath);
             }
-            catch (Exception ex) when (ex is DllNotFoundException || ex is BadImageFormatException)
+            catch (Exception ex) when (ex is DllNotFoundException or BadImageFormatException)
             {
                 Trace.TraceError($"LoadLibrary({extensionPath}) FAILED {ex}");
             }
@@ -85,7 +95,7 @@ namespace SOS.Extensions
             {
                 return HResult.E_FAIL;
             }
-            var initialializeCallback = SOSHost.GetDelegateFunction<InitializeCallbackDelegate>(extensionLibrary, "InitializeHostServices");
+            InitializeCallbackDelegate initialializeCallback = SOSHost.GetDelegateFunction<InitializeCallbackDelegate>(extensionLibrary, "InitializeHostServices");
             if (initialializeCallback == null)
             {
                 return HResult.E_FAIL;
@@ -97,19 +107,18 @@ namespace SOS.Extensions
 
         private HostServices()
         {
-            _serviceProvider = new ServiceProvider();
-            _symbolService = new SymbolService(this);
-            _commandProcessor = new CommandProcessor(RuntimeInformation.IsOSPlatform(OSPlatform.Windows) ? ">!ext" : null);
-            _commandProcessor.AddCommands(new Assembly[] { typeof(HostServices).Assembly });
-            _commandProcessor.AddCommands(new Assembly[] { typeof(ClrMDHelper).Assembly });
+            _serviceManager = new ServiceManager();
+            _commandService = new CommandService(RuntimeInformation.IsOSPlatform(OSPlatform.Windows) ? ">!ext" : null);
+            _serviceManager.NotifyExtensionLoad.Register(_commandService.AddCommands);
 
-            _serviceProvider.AddService<IHost>(this);
-            _serviceProvider.AddService<ICommandService>(_commandProcessor);
-            _serviceProvider.AddService<ISymbolService>(_symbolService);
+            _symbolService = new SymbolService(this)
+            {
+                DefaultTimeout = DefaultTimeout,
+                DefaultRetryCount = DefaultRetryCount
+            };
 
-            _hostWrapper = new HostWrapper(this, () => _targetWrapper);
-            _hostWrapper.AddServiceWrapper(IID_IHostServices, this);
-            _hostWrapper.AddServiceWrapper(SymbolServiceWrapper.IID_ISymbolService, () => new SymbolServiceWrapper(this, () => _memoryService));
+            _hostWrapper = new HostWrapper(this);
+            _hostWrapper.ServiceWrapper.AddServiceWrapper(IID_IHostServices, this);
 
             VTableBuilder builder = AddInterface(IID_IHostServices, validate: false);
             builder.AddMethod(new GetHostDelegate(GetHost));
@@ -119,7 +128,7 @@ namespace SOS.Extensions
             builder.AddMethod(new FlushTargetDelegate(FlushTarget));
             builder.AddMethod(new DestroyTargetDelegate(DestroyTarget));
             builder.AddMethod(new DispatchCommandDelegate(DispatchCommand));
-            builder.AddMethod(new DispatchCommandDelegate(DisplayHelp));
+            builder.AddMethod(new DisplayHelpDelegate(DisplayHelp));
             builder.AddMethod(new UninitializeDelegate(Uninitialize));
             IHostServices = builder.Complete();
 
@@ -129,47 +138,27 @@ namespace SOS.Extensions
         protected override void Destroy()
         {
             Trace.TraceInformation("HostServices.Destroy");
-            _hostWrapper.RemoveServiceWrapper(IID_IHostServices);
-            _hostWrapper.Release();
+            _hostWrapper.ServiceWrapper.RemoveServiceWrapper(IID_IHostServices);
+            _hostWrapper.ReleaseWithCheck();
         }
 
         #region IHost
 
         public IServiceEvent OnShutdownEvent { get; } = new ServiceEvent();
 
+        public IServiceEvent<ITarget> OnTargetCreate { get; } = new ServiceEvent<ITarget>();
+
         public HostType HostType => RuntimeInformation.IsOSPlatform(OSPlatform.Windows) ? HostType.DbgEng : HostType.Lldb;
 
-        IServiceProvider IHost.Services => _serviceProvider;
+        public IServiceProvider Services => _serviceContainer;
 
-        IEnumerable<ITarget> IHost.EnumerateTargets() => _target != null ? new ITarget[] { _target } : Array.Empty<ITarget>();
-
-        public void DestroyTarget(ITarget target)
-        {
-            if (target == null) {
-                throw new ArgumentNullException(nameof(target));
-            }
-            Trace.TraceInformation("IHost.DestroyTarget #{0}", target.Id);
-            if (target == _target)
-            {
-                _target = null;
-                _memoryService = null;
-                if (_targetWrapper != null)
-                {
-                    _targetWrapper.Release();
-                    _targetWrapper = null;
-                }
-                _contextService.ClearCurrentTarget();
-                if (target is IDisposable disposable) {
-                    disposable.Dispose();
-                }
-            }
-        }
+        public IEnumerable<ITarget> EnumerateTargets() => _target != null ? new ITarget[] { _target } : Array.Empty<ITarget>();
 
         #endregion
 
         #region IHostServices
 
-        private HResult GetHost(
+        private int GetHost(
             IntPtr self,
             out IntPtr host)
         {
@@ -178,12 +167,13 @@ namespace SOS.Extensions
             return HResult.S_OK;
         }
 
-        private HResult RegisterDebuggerServices(
+        private int RegisterDebuggerServices(
             IntPtr self,
             IntPtr iunk)
         {
             Trace.TraceInformation("HostServices.RegisterDebuggerServices");
-            if (iunk == IntPtr.Zero || DebuggerServices != null) {
+            if (iunk == IntPtr.Zero || DebuggerServices != null)
+            {
                 return HResult.E_FAIL;
             }
             // Create the wrapper for the host debugger services
@@ -196,31 +186,54 @@ namespace SOS.Extensions
                 Trace.TraceError(ex.Message);
                 return HResult.E_NOINTERFACE;
             }
-            try
-            {
-                var remoteMemoryService = new RemoteMemoryService(iunk);
-                _serviceProvider.AddService<IRemoteMemoryService>(remoteMemoryService);
-            }
-            catch (InvalidCastException)
-            {
-            }
             HResult hr;
             try
             {
-                var consoleService = new ConsoleServiceFromDebuggerServices(DebuggerServices);
-                _serviceProvider.AddService<IConsoleService>(consoleService);
+                ConsoleServiceFromDebuggerServices consoleService = new(DebuggerServices);
+                FileLoggingConsoleService fileLoggingConsoleService = new(consoleService);
+                DiagnosticLoggingService.Instance.SetConsole(consoleService, fileLoggingConsoleService);
+
+                // Don't register everything in the SOSHost assembly; just the wrappers
+                _serviceManager.RegisterExportedServices(typeof(TargetWrapper));
+                _serviceManager.RegisterExportedServices(typeof(RuntimeWrapper));
+
+                // Register all the services and commands in the Microsoft.Diagnostics.DebugServices.Implementation assembly
+                _serviceManager.RegisterAssembly(typeof(Target).Assembly);
+
+                // Register all the services and commands in the SOS.Extensions (this) assembly
+                _serviceManager.RegisterAssembly(typeof(HostServices).Assembly);
+
+                // Register all the services and commands in the Microsoft.Diagnostics.ExtensionCommands assembly
+                _serviceManager.RegisterAssembly(typeof(ClrMDHelper).Assembly);
+
+                // Display any extension assembly loads on console
+                _serviceManager.NotifyExtensionLoad.Register((Assembly assembly) => fileLoggingConsoleService.WriteLine($"Loading extension {assembly.Location}"));
+                _serviceManager.NotifyExtensionLoadFailure.Register((Exception ex) => fileLoggingConsoleService.WriteLine(ex.Message));
+
+                // Load any extra extensions in the search path
+                _serviceManager.LoadExtensions();
+
+                // Loading extensions or adding service factories not allowed after this point.
+                _serviceManager.FinalizeServices();
+
+                // Add all the global services to the global service container
+                _serviceContainer = _serviceManager.CreateServiceContainer(ServiceScope.Global, parent: null);
+                _serviceContainer.AddService<IServiceManager>(_serviceManager);
+                _serviceContainer.AddService<IHost>(this);
+                _serviceContainer.AddService<ICommandService>(_commandService);
+                _serviceContainer.AddService<ISymbolService>(_symbolService);
+                _serviceContainer.AddService<IConsoleService>(fileLoggingConsoleService);
+                _serviceContainer.AddService<IConsoleFileLoggingService>(fileLoggingConsoleService);
+                _serviceContainer.AddService<IDiagnosticLoggingService>(DiagnosticLoggingService.Instance);
 
                 _contextService = new ContextServiceFromDebuggerServices(this, DebuggerServices);
-                _serviceProvider.AddService<IContextService>(_contextService);
-                _serviceProvider.AddServiceFactory<IThreadUnwindService>(() => new ThreadUnwindServiceFromDebuggerServices(DebuggerServices));
+                _serviceContainer.AddService<IContextService>(_contextService);
 
-                _contextService.ServiceProvider.AddServiceFactory<ClrMDHelper>(() => {
-                    ClrRuntime clrRuntime = _contextService.Services.GetService<ClrRuntime>();
-                    return clrRuntime != null ? new ClrMDHelper(clrRuntime) : null;
-                });
+                ThreadUnwindServiceFromDebuggerServices threadUnwindService = new(DebuggerServices);
+                _serviceContainer.AddService<IThreadUnwindService>(threadUnwindService);
 
                 // Add each extension command to the native debugger
-                foreach ((string name, string help, IEnumerable<string> aliases) in _commandProcessor.Commands)
+                foreach ((string name, string help, IEnumerable<string> aliases) in _commandService.Commands)
                 {
                     hr = DebuggerServices.AddCommand(name, help, aliases);
                     if (hr != HResult.S_OK)
@@ -228,12 +241,29 @@ namespace SOS.Extensions
                         Trace.TraceWarning($"Cannot add extension command {hr:X8} {name} - {help}");
                     }
                 }
+
+                if (DebuggerServices.DebugClient is IDebugControl5 control)
+                {
+                    MemoryRegionServiceFromDebuggerServices memRegions = new(DebuggerServices.DebugClient, control);
+                    _serviceContainer.AddService<IMemoryRegionService>(memRegions);
+                }
             }
             catch (Exception ex)
             {
                 Trace.TraceError(ex.ToString());
                 return HResult.E_FAIL;
             }
+            try
+            {
+                RemoteMemoryService remoteMemoryService = new(iunk);
+                // This service needs another reference since it is implemented as part of IDebuggerServices and gets
+                // disposed in Uninitialize() below by the DisposeServices call.
+                remoteMemoryService.AddRef();
+                _serviceContainer.AddService<IRemoteMemoryService>(remoteMemoryService);
+            }
+            catch (InvalidCastException)
+            {
+            }
             hr = DebuggerServices.GetSymbolPath(out string symbolPath);
             if (hr == HResult.S_OK)
             {
@@ -249,19 +279,19 @@ namespace SOS.Extensions
             return HResult.S_OK;
         }
 
-        private HResult CreateTarget(
+        private int CreateTarget(
             IntPtr self)
         {
             Trace.TraceInformation("HostServices.CreateTarget");
-            if (_target != null || DebuggerServices == null) {
+            if (_target != null || DebuggerServices == null)
+            {
                 return HResult.E_FAIL;
             }
             try
             {
-                _target = new TargetFromDebuggerServices(DebuggerServices, this, _targetIdFactory++);
-                _contextService.SetCurrentTarget(_target);
-                _targetWrapper = new TargetWrapper(_contextService.Services);
-                _memoryService = _contextService.Services.GetService<IMemoryService>();
+                TargetFromDebuggerServices target = new(DebuggerServices, this, _targetIdFactory++);
+                _contextService.SetCurrentTarget(target);
+                _target = target;
             }
             catch (Exception ex)
             {
@@ -271,7 +301,7 @@ namespace SOS.Extensions
             return HResult.S_OK;
         }
 
-        private HResult UpdateTarget(
+        private int UpdateTarget(
             IntPtr self,
             uint processId)
         {
@@ -292,33 +322,54 @@ namespace SOS.Extensions
             IntPtr self)
         {
             Trace.TraceInformation("HostServices.FlushTarget");
-            if (_target != null)
-            {
-                _target.Flush();
-            }
+            _target?.Flush();
         }
 
         private void DestroyTarget(
             IntPtr self)
         {
             Trace.TraceInformation("HostServices.DestroyTarget #{0}", _target != null ? _target.Id : "<none>");
-            if (_target != null)
+            try
             {
-                DestroyTarget(_target);
+                _target?.Destroy();
+                _target = null;
+            }
+            catch (Exception ex)
+            {
+                Trace.TraceError(ex.ToString());
             }
         }
 
-        private HResult DispatchCommand(
+        private int DispatchCommand(
             IntPtr self,
-            string commandLine)
+            string commandName,
+            string commandArguments)
         {
-            if (commandLine == null)
+            if (string.IsNullOrWhiteSpace(commandName))
             {
                 return HResult.E_INVALIDARG;
             }
+            if (!_commandService.IsCommand(commandName))
+            {
+                return HResult.E_NOTIMPL;
+            }
             try
             {
-                return _commandProcessor.Execute(commandLine, _contextService.Services);
+                StringBuilder sb = new();
+                sb.Append(commandName);
+                if (!string.IsNullOrWhiteSpace(commandArguments))
+                {
+                    sb.Append(' ');
+                    sb.Append(commandArguments);
+                }
+                if (_commandService.Execute(sb.ToString(), _contextService.Services))
+                {
+                    return HResult.S_OK;
+                }
+            }
+            catch (CommandNotSupportedException)
+            {
+                return HResult.E_NOTIMPL;
             }
             catch (Exception ex)
             {
@@ -327,17 +378,21 @@ namespace SOS.Extensions
             return HResult.E_FAIL;
         }
 
-        private HResult DisplayHelp(
+        private int DisplayHelp(
             IntPtr self,
-            string command)
+            string commandName)
         {
             try
             {
-                if (!_commandProcessor.DisplayHelp(command, _contextService.Services))
+                if (!_commandService.DisplayHelp(commandName, _contextService.Services))
                 {
                     return HResult.E_INVALIDARG;
                 }
             }
+            catch (CommandNotSupportedException)
+            {
+                return HResult.E_NOTIMPL;
+            }
             catch (Exception ex)
             {
                 Trace.TraceError(ex.ToString());
@@ -350,44 +405,55 @@ namespace SOS.Extensions
             IntPtr self)
         {
             Trace.TraceInformation("HostServices.Uninitialize");
-            DestroyTarget(self);
-
-            if (DebuggerServices != null)
+            try
             {
-                DebuggerServices.Release();
+                DestroyTarget(self);
+
+                // Send shutdown event on exit
+                OnShutdownEvent.Fire();
+
+                // Dispose of the global services which RemoteMemoryService but not host services (this)
+                _serviceContainer.DisposeServices();
+
+                // This turns off any logging to console now that debugger services will be released and the console service will no longer work.
+                DiagnosticLoggingService.Instance.SetConsole(consoleService: null, fileLoggingService: null);
+
+                // Release the debugger services instance
+                DebuggerServices?.ReleaseWithCheck();
                 DebuggerServices = null;
+
+                // Clear HostService instance
+                Instance = null;
+
+                // Release the host services wrapper
+                this.ReleaseWithCheck();
+            }
+            catch (Exception ex)
+            {
+                Trace.TraceError(ex.ToString());
             }
-
-            // Send shutdown event on exit
-            OnShutdownEvent.Fire();
-
-            // Release the host services wrapper
-            Release();
-
-            // Clear HostService instance
-            Instance = null;
         }
 
         #endregion
-        
+
         #region IHostServices delegates
 
         [UnmanagedFunctionPointer(CallingConvention.Winapi)]
-        private delegate HResult GetHostDelegate(
+        private delegate int GetHostDelegate(
             [In] IntPtr self,
             [Out] out IntPtr host);
 
         [UnmanagedFunctionPointer(CallingConvention.Winapi)]
-        private delegate HResult RegisterDebuggerServicesDelegate(
+        private delegate int RegisterDebuggerServicesDelegate(
             [In] IntPtr self,
             [In] IntPtr iunk);
 
         [UnmanagedFunctionPointer(CallingConvention.Winapi)]
-        private delegate HResult CreateTargetDelegate(
+        private delegate int CreateTargetDelegate(
             [In] IntPtr self);
 
         [UnmanagedFunctionPointer(CallingConvention.Winapi)]
-        private delegate HResult UpdateTargetDelegate(
+        private delegate int UpdateTargetDelegate(
             [In] IntPtr self,
             [In] uint processId);
 
@@ -400,14 +466,15 @@ namespace SOS.Extensions
             [In] IntPtr self);
 
         [UnmanagedFunctionPointer(CallingConvention.Winapi)]
-        private delegate HResult DispatchCommandDelegate(
+        private delegate int DispatchCommandDelegate(
             [In] IntPtr self,
-            [In, MarshalAs(UnmanagedType.LPStr)] string commandLine);
+            [In, MarshalAs(UnmanagedType.LPStr)] string commandName,
+            [In, MarshalAs(UnmanagedType.LPStr)] string commandArguments);
 
         [UnmanagedFunctionPointer(CallingConvention.Winapi)]
-        private delegate HResult DisplayHelpDelegate(
+        private delegate int DisplayHelpDelegate(
             [In] IntPtr self,
-            [In, MarshalAs(UnmanagedType.LPStr)] string command);
+            [In, MarshalAs(UnmanagedType.LPStr)] string commandName);
 
         [UnmanagedFunctionPointer(CallingConvention.Winapi)]
         private delegate void UninitializeDelegate(
