@@ -1,6 +1,5 @@
 // Licensed to the .NET Foundation under one or more agreements.
 // The .NET Foundation licenses this file to you under the MIT license.
-// See the LICENSE file in the project root for more information.
 
 #pragma once
 
@@ -46,7 +45,7 @@ namespace sos
     public:
         Exception(const char *format, va_list args)
         {
-            vsprintf_s(mMsg, _countof(mMsg), format, args);
+            vsprintf_s(mMsg, ARRAY_SIZE(mMsg), format, args);
 
             va_end(args);
         }
@@ -627,8 +626,27 @@ namespace sos
         void BuildError(__out_ecount(count) char *out, size_t count, const char *format, ...) const;
 
         void AssertSanity() const;
-        bool NextSegment();
-        bool CheckSegmentRange();
+
+        /*
+            This function moves to the next segment/region without checking any restrictions
+            on the range. Returns true if it was able to move to a new segment/region.
+        */
+        bool TryMoveNextSegment();
+
+        /*
+            Aligns the iterator to the object that falls in the requested range, moving to 
+            the next segment/region as necessary. The iterator state doesn't change if the
+            current object already lies in the requested range. Returns true if aligning
+            to such an object was possible.
+        */
+        bool TryAlignToObjectInRange();
+
+        /*
+            Moves to the next segment/region that contains an object in the requested
+            range and align it to such object. This operation always moves the iterator.
+            Returns false if no such move was possible.
+        */
+        bool TryMoveToObjectInNextSegmentInRange();
         void MoveToNextObject();
 
     private:
@@ -721,7 +739,7 @@ namespace sos
         inline const SyncBlkIterator &operator++()
         {
             SOS_Assert(mCurr <= mTotal);
-            mSyncBlk = ++mCurr;
+            mSyncBlk = mCurr++;
 
             return *this;
         }
