@@ -1,6 +1,5 @@
 // Licensed to the .NET Foundation under one or more agreements.
 // The .NET Foundation licenses this file to you under the MIT license.
-// See the LICENSE file in the project root for more information.
 
 // ==++==
 // 
@@ -92,8 +91,8 @@ const char *getFacilityName(DWORD_PTR lf)
         {
             if ( lf & 0x1 )
             {
-                strcat_s ( buff, _countof(buff), &(facilities[i].lfName[3]) );
-                strcat_s ( buff, _countof(buff), "`" );
+                strcat_s ( buff, ARRAY_SIZE(buff), &(facilities[i].lfName[3]) );
+                strcat_s ( buff, ARRAY_SIZE(buff), "`" );
             }
             lf >>= 1;
         }
@@ -121,7 +120,7 @@ void formatOutput(struct IDebugDataSpaces* memCallBack, ___in FILE* file, __inou
     
     int iArgCount = 0;
     
-    strcpy_s(formatCopy, _countof(formatCopy), format);
+    strcpy_s(formatCopy, ARRAY_SIZE(formatCopy), format);
     for(;;) 
     {
         char c = *ptr++;
@@ -178,7 +177,7 @@ void formatOutput(struct IDebugDataSpaces* memCallBack, ___in FILE* file, __inou
                                     static WCHAR wszNameBuffer[1024]; // should be large enough
                                     if (g_sos->GetMethodDescName(arg, 1024, wszNameBuffer, NULL) != S_OK)
                                     {
-                                        wcscpy_s(wszNameBuffer, _countof(wszNameBuffer), W("UNKNOWN METHODDESC"));
+                                        wcscpy_s(wszNameBuffer, ARRAY_SIZE(wszNameBuffer), W("UNKNOWN METHODDESC"));
                                     }
 
                                     wcscpy_s(buff, capacity_buff, wszNameBuffer);
@@ -513,7 +512,7 @@ HRESULT StressLog::Dump(ULONG64 outProcLog, const char* fileName, struct IDebugD
             TADDR taFmt = GetFormatAddr(inProcLog, latestMsg->formatOffset, bHasModuleTable);
             hr = memCallBack->ReadVirtual(TO_CDADDR(taFmt), format, 256, 0);
             if (hr != S_OK) 
-                strcpy_s(format, _countof(format), "Could not read address of format string");
+                strcpy_s(format, ARRAY_SIZE(format), "Could not read address of format string");
 
             double deltaTime = ((double) (latestMsg->timeStamp - inProcLog.startTimeStamp)) / inProcLog.tickFrequency;
             if (bDoGcHist)
@@ -575,4 +574,3 @@ FREE_MEM:
 
     return hr;
 }
-
