@@ -1,11 +1,13 @@
 // Copyright (c) .NET Foundation. All rights reserved.
 // Licensed under the Apache License, Version 2.0. See License.txt in the project root for license information.
 
+using System.Threading.Tasks;
+
 namespace NuGetConsole
 {
     internal interface IHostInitializer
     {
-        void Start();
+        Task StartAsync();
 
         void SetDefaultRunspace();
     }
