@@ -13,7 +13,6 @@ using Microsoft;
 using Microsoft.Build.Evaluation;
 using Microsoft.CSharp.RuntimeBinder;
 using Microsoft.VisualStudio;
-using Microsoft.VisualStudio.ProjectSystem.Interop;
 using Microsoft.VisualStudio.Shell;
 using Microsoft.VisualStudio.Shell.Interop;
 using NuGet.Common;
@@ -45,9 +44,10 @@ namespace NuGet.PackageManagement.VisualStudio
 
         #region Get Project Information
 
-        internal static async Task<bool> IsSolutionFolderAsync(EnvDTE.Project envDTEProject)
+        internal static bool IsSolutionFolder(EnvDTE.Project envDTEProject)
         {
-            await NuGetUIThreadHelper.JoinableTaskFactory.SwitchToMainThreadAsync();
+            ThreadHelper.ThrowIfNotOnUIThread();
+
             return envDTEProject.Kind != null && envDTEProject.Kind.Equals(VsProjectTypes.VsProjectItemKindSolutionFolder, StringComparison.OrdinalIgnoreCase);
         }
 
@@ -75,26 +75,26 @@ namespace NuGet.PackageManagement.VisualStudio
 
             await NuGetUIThreadHelper.JoinableTaskFactory.SwitchToMainThreadAsync();
 
-            var vsProject = (IVsProject)(await envDTEProject.ToVsHierarchyAsync());
+            var hierarchy = await envDTEProject.ToVsHierarchyAsync();
+            var vsProject = hierarchy as IVsProject;
             if (vsProject == null)
             {
                 return false;
             }
 
             int pFound;
-            uint itemId;
 
-            if (await IsProjectCapabilityCompliantAsync(envDTEProject))
+            if (VsHierarchyUtility.IsProjectCapabilityCompliant(hierarchy))
             {
                 // REVIEW: We want to revisit this after RTM - the code in this if statement should be applied to every project type.
                 // We're checking for VSDOCUMENTPRIORITY.DP_Standard here to see if the file is included in the project.
                 // Original check (outside of if) did not have this.
                 var priority = new VSDOCUMENTPRIORITY[1];
-                var hr = vsProject.IsDocumentInProject(path, out pFound, priority, out itemId);
+                var hr = vsProject.IsDocumentInProject(path, out pFound, priority, out _);
                 return ErrorHandler.Succeeded(hr) && pFound == 1 && priority[0] >= VSDOCUMENTPRIORITY.DP_Standard;
             }
 
-            var hres = vsProject.IsDocumentInProject(path, out pFound, Array.Empty<VSDOCUMENTPRIORITY>(), out itemId);
+            var hres = vsProject.IsDocumentInProject(path, out pFound, Array.Empty<VSDOCUMENTPRIORITY>(), out _);
             return ErrorHandler.Succeeded(hres) && pFound == 1;
         }
 
@@ -390,22 +390,14 @@ namespace NuGet.PackageManagement.VisualStudio
         public static async Task<bool> IsSupportedAsync(EnvDTE.Project envDTEProject)
         {
             Assumes.Present(envDTEProject);
-
-            if (await IsProjectCapabilityCompliantAsync(envDTEProject))
+            await NuGetUIThreadHelper.JoinableTaskFactory.SwitchToMainThreadAsync();
+            var hierarchy = await envDTEProject.ToVsHierarchyAsync();
+            if (VsHierarchyUtility.IsProjectCapabilityCompliant(hierarchy))
             {
                 return true;
             }
 
-            await NuGetUIThreadHelper.JoinableTaskFactory.SwitchToMainThreadAsync();
-
-            return envDTEProject.Kind != null && ProjectType.IsSupported(envDTEProject.Kind) && !await HasUnsupportedProjectCapabilityAsync(envDTEProject);
-        }
-
-        private static async Task<bool> IsProjectCapabilityCompliantAsync(EnvDTE.Project envDTEProject)
-        {
-            Debug.Assert(envDTEProject != null);
-            var hierarchy = await envDTEProject.ToVsHierarchyAsync();
-            return await VsHierarchyUtility.IsProjectCapabilityCompliantAsync(hierarchy);
+            return envDTEProject.Kind != null && ProjectType.IsSupported(envDTEProject.Kind) && !VsHierarchyUtility.HasUnsupportedProjectCapability(hierarchy);
         }
 
         public async static Task<NuGetProject> GetNuGetProjectAsync(EnvDTE.Project project, ISolutionManager solutionManager)
@@ -442,7 +434,7 @@ namespace NuGet.PackageManagement.VisualStudio
         {
             await NuGetUIThreadHelper.JoinableTaskFactory.SwitchToMainThreadAsync();
 
-            if (!await IsSolutionFolderAsync(envDTEProject))
+            if (!IsSolutionFolder(envDTEProject))
             {
                 return Array.Empty<EnvDTE.Project>();
             }
@@ -464,7 +456,7 @@ namespace NuGet.PackageManagement.VisualStudio
                     {
                         supportedChildProjects.Add(nestedProject);
                     }
-                    else if (await IsSolutionFolderAsync(nestedProject))
+                    else if (IsSolutionFolder(nestedProject))
                     {
                         containerProjects.Enqueue(nestedProject);
                     }
