@@ -30,7 +30,7 @@ namespace NuGet.VisualStudio
             IVsHierarchy hierarchy;
 
             // Get the vs solution
-            var solution = await ServiceLocator.GetInstanceAsync<IVsSolution>();
+            var solution = await ServiceLocator.GetGlobalServiceAsync<SVsSolution, IVsSolution>();
             int hr = solution.GetProjectOfUniqueName(project.GetUniqueName(), out hierarchy);
 
             if (hr != VSConstants.S_OK)
@@ -49,9 +49,7 @@ namespace NuGet.VisualStudio
 
             // Get the vs hierarchy as an IVsAggregatableProject to get the project type guids
             IVsHierarchy hierarchy = await ToVsHierarchyAsync(project);
-            string[] projectTypeGuids = VsHierarchyUtility.GetProjectTypeGuids(hierarchy, project.Kind);
-
-            return projectTypeGuids;
+            return VsHierarchyUtility.GetProjectTypeGuidsFromHierarchy(hierarchy);
         }
 
         #region Constants
