@@ -4,8 +4,10 @@
 using System;
 using System.Collections.Generic;
 using System.ComponentModel.Composition;
+using System.Globalization;
 using System.IO;
 using System.Linq;
+using System.Runtime.InteropServices;
 using System.Xml.Linq;
 using EnvDTE;
 using Microsoft.VisualStudio;
@@ -19,6 +21,7 @@ using NuGet.PackageManagement.VisualStudio;
 using NuGet.ProjectManagement;
 using NuGet.Protocol.Core.Types;
 using NuGet.Versioning;
+using NuGet.VisualStudio.Implementation.Extensibility;
 using NuGet.VisualStudio.Implementation.Resources;
 using Task = System.Threading.Tasks.Task;
 using XmlUtility = NuGet.Shared.XmlUtility;
@@ -248,7 +251,7 @@ namespace NuGet.VisualStudio
                     return RepositoryType.Template;
 
                 default:
-                    ShowErrorMessage(String.Format(VsResources.TemplateWizard_InvalidRepositoryAttribute,
+                    ShowErrorMessage(string.Format(CultureInfo.CurrentCulture, VsResources.TemplateWizard_InvalidRepositoryAttribute,
                         repositoryAttributeValue));
                     throw new WizardBackoutException();
             }
@@ -282,7 +285,7 @@ namespace NuGet.VisualStudio
                     var packageManagementFormat = new PackageManagementFormat(_settings);
                     // 1 means PackageReference
                     var preferPackageReference = packageManagementFormat.SelectedPackageManagementFormat == 1;
-                    await PreinstalledPackageInstaller.PerformPackageInstallAsync(_installer,
+                    await PreinstalledPackageInstaller.PerformPackageInstallAsync(
                         project,
                         configuration,
                         preferPackageReference,
@@ -357,6 +360,33 @@ namespace NuGet.VisualStudio
             }
         }
 
+        private string GetSolutionDirectoryFromDte(DTE dte)
+        {
+            ThreadHelper.ThrowIfNotOnUIThread();
+            // Use .Properties.Item("Path") instead of .FullName because .FullName might not be
+            // available if the solution is just being created
+            string solutionFilePath;
+
+            var property = dte.Solution.Properties.Item("Path");
+            if (property == null)
+            {
+                return null;
+            }
+            try
+            {
+                // When using a temporary solution, (such as by saying File -> New File), querying this value throws.
+                // Since we wouldn't be able to do manage any packages at this point, we return null. Consumers of this property typically
+                // use a String.IsNullOrEmpty check either way, so it's alright.
+                solutionFilePath = (string)property.Value;
+            }
+            catch (COMException)
+            {
+                return null;
+            }
+
+            return Path.GetDirectoryName(solutionFilePath);
+        }
+
         private void AddTemplateParameters(Dictionary<string, string> replacementsDictionary)
         {
             ThreadHelper.ThrowIfNotOnUIThread();
@@ -371,8 +401,8 @@ namespace NuGet.VisualStudio
                 if (_dte.Solution != null
                     && _dte.Solution.IsOpen)
                 {
-                    //solutionRepositoryPath = RepositorySettings.Value.RepositoryPath;
-                    solutionRepositoryPath = PackagesFolderPathUtility.GetPackagesFolderPath(_solutionManager, _settings);
+                    var solutionDirectory = GetSolutionDirectoryFromDte(_dte);
+                    solutionRepositoryPath = PackagesFolderPathUtility.GetPackagesFolderPath(solutionDirectory, _settings);
                 }
                 else
                 {
@@ -413,7 +443,7 @@ namespace NuGet.VisualStudio
             }
 
             // provide a current timpestamp (for use by universal provider)
-            replacementsDictionary["$timestamp$"] = DateTime.Now.ToString("yyyyMMddHHmmss");
+            replacementsDictionary["$timestamp$"] = DateTime.Now.ToString("yyyyMMddHHmmss", CultureInfo.CurrentCulture);
         }
 
         internal virtual void ThrowWizardBackoutError(string message)
