@@ -12,6 +12,7 @@ using NuGet.LibraryModel;
 using NuGet.Packaging;
 using NuGet.ProjectModel;
 using NuGet.Repositories;
+using NuGet.Shared;
 using NuGetVersion = NuGet.Versioning.NuGetVersion;
 
 namespace NuGet.Commands
@@ -190,9 +191,6 @@ namespace NuGet.Commands
                     && (target.TargetFramework is FallbackFramework
                         || target.TargetFramework is AssetTargetFallbackFramework);
 
-                // This is used for a special-case warning we need to do when net6.0-maccatalyst selects a xamarin.ios asset.
-                MaccatalystFallback maccatalystFallback = MaccatalystFallback.FallbackIfNeeded(targetGraph.Framework);
-
                 foreach (var graphItem in targetGraph.Flattened.OrderBy(x => x.Key))
                 {
                     var library = graphItem.Key;
@@ -217,8 +215,7 @@ namespace NuGet.Commands
                             library,
                             includeFlags,
                             targetGraph,
-                            rootProjectStyle,
-                            maccatalystFallback);
+                            rootProjectStyle);
 
                         target.Libraries.Add(projectLib);
                     }
@@ -234,7 +231,7 @@ namespace NuGet.Commands
                         var package = packageInfo.Package;
                         var libraryDependency = tfi.Dependencies.FirstOrDefault(e => e.Name.Equals(library.Name, StringComparison.OrdinalIgnoreCase));
 
-                        var targetLibrary = LockFileUtils.CreateLockFileTargetLibrary(
+                        (LockFileTargetLibrary targetLibrary, bool usedFallbackFramework) = LockFileUtils.CreateLockFileTargetLibrary(
                             libraryDependency?.Aliases,
                             libraries[Tuple.Create(library.Name, library.Version)],
                             package,
@@ -242,28 +239,31 @@ namespace NuGet.Commands
                             dependencyType: includeFlags,
                             targetFrameworkOverride: null,
                             dependencies: graphItem.Data.Dependencies,
-                            cache: lockFileBuilderCache,
-                            maccatalystFallback);
+                            cache: lockFileBuilderCache);
 
                         target.Libraries.Add(targetLibrary);
 
                         // Log warnings if the target library used the fallback framework
                         if (warnForImportsOnGraph && !librariesWithWarnings.Contains(library))
                         {
-                            var nonFallbackFramework = new NuGetFramework(target.TargetFramework);
+                            if (target.TargetFramework is FallbackFramework)
+                            {
+                                // PackageTargetFallback works different from AssetTargetFallback so the warning logic for PTF cannot be optimized.
+                                var nonFallbackFramework = new NuGetFramework(target.TargetFramework);
 
-                            var targetLibraryWithoutFallback = LockFileUtils.CreateLockFileTargetLibrary(
-                                libraryDependency?.Aliases,
-                                libraries[Tuple.Create(library.Name, library.Version)],
-                                package,
-                                targetGraph,
-                                targetFrameworkOverride: nonFallbackFramework,
-                                dependencyType: includeFlags,
-                                dependencies: graphItem.Data.Dependencies,
-                                cache: lockFileBuilderCache,
-                                maccatalystFallback: maccatalystFallback);
+                                var targetLibraryWithoutFallback = LockFileUtils.CreateLockFileTargetLibrary(
+                                    libraryDependency?.Aliases,
+                                    libraries[Tuple.Create(library.Name, library.Version)],
+                                    package,
+                                    targetGraph,
+                                    targetFrameworkOverride: nonFallbackFramework,
+                                    dependencyType: includeFlags,
+                                    dependencies: graphItem.Data.Dependencies,
+                                    cache: lockFileBuilderCache);
+                                usedFallbackFramework = !targetLibrary.Equals(targetLibraryWithoutFallback);
+                            }
 
-                            if (!targetLibrary.Equals(targetLibraryWithoutFallback))
+                            if (usedFallbackFramework)
                             {
                                 var libraryName = DiagnosticUtility.FormatIdentity(library);
 
@@ -271,7 +271,7 @@ namespace NuGet.Commands
                                     Strings.Log_ImportsFallbackWarning,
                                     libraryName,
                                     GetFallbackFrameworkString(target.TargetFramework),
-                                    nonFallbackFramework);
+                                    new NuGetFramework(target.TargetFramework));
 
                                 var logMessage = RestoreLogMessage.CreateWarning(
                                     NuGetLogCode.NU1701,
@@ -286,29 +286,6 @@ namespace NuGet.Commands
                             }
                         }
                     }
-
-                    if (maccatalystFallback != null && maccatalystFallback._usedXamarinIOs)
-                    {
-                        var libraryName = DiagnosticUtility.FormatIdentity(library);
-
-                        var message = string.Format(CultureInfo.CurrentCulture,
-                            Strings.Warning_MacCatalystXamarinIOSCompat,
-                            libraryName,
-                            project.Name,
-                            targetGraph.Framework);
-
-                        var logMessage = RestoreLogMessage.CreateWarning(
-                            NuGetLogCode.NU1703,
-                            message,
-                            library.Name,
-                            targetGraph.TargetGraphName);
-
-                        _logger.Log(logMessage);
-
-                        // only log the warning once per library
-                        librariesWithWarnings.Add(library);
-                    }
-
                 }
 
                 EnsureUniqueLockFileTargetLibraries(target);
@@ -360,26 +337,24 @@ namespace NuGet.Commands
             return libraryReferences;
         }
 
-        private void EnsureUniqueLockFileTargetLibraries(LockFileTarget lockFileTarget)
+        private static void EnsureUniqueLockFileTargetLibraries(LockFileTarget lockFileTarget)
         {
             IList<LockFileTargetLibrary> libraries = lockFileTarget.Libraries;
-            var libraryReferences = new Dictionary<string, LockFileTargetLibrary>();
+            var libraryReferences = new Dictionary<LockFileTargetLibrary, LockFileTargetLibrary>(comparer: LockFileTargetLibraryNameAndVersionEqualityComparer.Instance);
 
             foreach (LockFileTargetLibrary library in libraries)
             {
-                var libraryKey = library.Name + " " + library.Version;
-
-                if (libraryReferences.TryGetValue(libraryKey, out LockFileTargetLibrary existingLibrary))
+                if (libraryReferences.TryGetValue(library, out LockFileTargetLibrary existingLibrary))
                 {
                     if (RankReferences(existingLibrary.Type) > RankReferences(library.Type))
                     {
                         // Prefer project reference over package reference, so replace the the package reference.
-                        libraryReferences[libraryKey] = library;
+                        libraryReferences[library] = library;
                     }
                 }
                 else
                 {
-                    libraryReferences[libraryKey] = library;
+                    libraryReferences[library] = library;
                 }
             }
 
@@ -389,7 +364,7 @@ namespace NuGet.Commands
             }
 
             lockFileTarget.Libraries = new List<LockFileTargetLibrary>(libraryReferences.Count);
-            foreach (KeyValuePair<string, LockFileTargetLibrary> pair in libraryReferences)
+            foreach (KeyValuePair<LockFileTargetLibrary, LockFileTargetLibrary> pair in libraryReferences)
             {
                 lockFileTarget.Libraries.Add(pair.Value);
             }
@@ -499,50 +474,141 @@ namespace NuGet.Commands
 
         private void AddCentralTransitiveDependencyGroupsForPackageReference(PackageSpec project, LockFile lockFile, IEnumerable<RestoreTargetGraph> targetGraphs)
         {
-            if (project.RestoreMetadata?.CentralPackageVersionsEnabled == false)
+            if (project.RestoreMetadata == null || !project.RestoreMetadata.CentralPackageVersionsEnabled || !project.RestoreMetadata.CentralPackageTransitivePinningEnabled)
             {
                 return;
             }
 
             // Do not pack anything from the runtime graphs
             // The runtime graphs are added in addition to the graphs without a runtime
-            foreach (var targetGraph in targetGraphs.Where(targetGraph => string.IsNullOrEmpty(targetGraph.RuntimeIdentifier)))
+            foreach (RestoreTargetGraph targetGraph in targetGraphs.Where(targetGraph => string.IsNullOrEmpty(targetGraph.RuntimeIdentifier)))
             {
-                var centralPackageVersionsForFramework = project.TargetFrameworks.Where(tfmi => tfmi.FrameworkName.Equals(targetGraph.Framework)).FirstOrDefault()?.CentralPackageVersions;
+                TargetFrameworkInformation targetFrameworkInformation = project.TargetFrameworks.FirstOrDefault(i => i.FrameworkName.Equals(targetGraph.Framework));
+
+                if (targetFrameworkInformation == null)
+                {
+                    continue;
+                }
 
                 // The transitive dependencies enforced by the central package version management file are written to the assets to be used by the pack task.
-                IEnumerable<LibraryDependency> centralEnforcedTransitiveDependencies = targetGraph
-                    .Flattened
-                    .Where(graphItem => graphItem.IsCentralTransitive && centralPackageVersionsForFramework?.ContainsKey(graphItem.Key.Name) == true)
-                    .Select((graphItem) =>
-                    {
-                        CentralPackageVersion matchingCentralVersion = centralPackageVersionsForFramework[graphItem.Key.Name];
-                        Dictionary<string, LibraryIncludeFlags> dependenciesIncludeFlags = _includeFlagGraphs[targetGraph];
-
-                        var libraryDependency = new LibraryDependency()
-                        {
-                            LibraryRange = new LibraryRange(matchingCentralVersion.Name, matchingCentralVersion.VersionRange, LibraryDependencyTarget.Package),
-                            ReferenceType = LibraryDependencyReferenceType.Transitive,
-                            VersionCentrallyManaged = true,
-                            IncludeType = dependenciesIncludeFlags[matchingCentralVersion.Name]
-                        };
-
-                        return libraryDependency;
-                    });
+                List<LibraryDependency> centralEnforcedTransitiveDependencies = GetLibraryDependenciesForCentralTransitiveDependencies(targetGraph, targetFrameworkInformation).ToList();
 
                 if (centralEnforcedTransitiveDependencies.Any())
                 {
                     var centralEnforcedTransitiveDependencyGroup = new CentralTransitiveDependencyGroup
-                            (
-                                targetGraph.Framework,
-                                centralEnforcedTransitiveDependencies
-                            );
+                    (
+                        targetGraph.Framework,
+                        centralEnforcedTransitiveDependencies
+                    );
 
                     lockFile.CentralTransitiveDependencyGroups.Add(centralEnforcedTransitiveDependencyGroup);
                 }
             }
         }
 
+        /// <summary>
+        /// Determines the <see cref="LibraryDependency" /> objects for the specified <see cref="RestoreTargetGraph" /> that represent the centrally defined transitive dependencies.
+        /// </summary>
+        /// <param name="targetGraph">The <see cref="RestoreTargetGraph" /> to get centrally defined transitive dependencies for.</param>
+        /// <param name="targetFrameworkInformation">The <see cref="TargetFrameworkInformation" /> for the target framework to get centrally defined transitive dependencies for.</param>
+        /// <param name="centralPackageTransitivePinningEnabled">A value indicating whether or not central transitive dependency version pinning is enabled.</param>
+        /// <returns>An <see cref="IEnumerable{LibraryDependency}" /> representing the centrally defined transitive dependencies for the specified <see cref="RestoreTargetGraph" />.</returns>
+        private IEnumerable<LibraryDependency> GetLibraryDependenciesForCentralTransitiveDependencies(RestoreTargetGraph targetGraph, TargetFrameworkInformation targetFrameworkInformation)
+        {
+            HashSet<GraphNode<RemoteResolveResult>> visitedNodes = new HashSet<GraphNode<RemoteResolveResult>>();
+            Queue<GraphNode<RemoteResolveResult>> queue = new Queue<GraphNode<RemoteResolveResult>>();
+
+            foreach (GraphNode<RemoteResolveResult> rootNode in targetGraph.Graphs)
+            {
+                Dictionary<string, LibraryDependency> dependencyDictionary = null;
+
+                foreach (GraphNode<RemoteResolveResult> node in rootNode.InnerNodes)
+                {
+                    // Only consider nodes that are Accepted, IsCentralTransitive, and have a centrally defined package version
+                    if (node?.Item == null || node.Disposition != Disposition.Accepted || !node.Item.IsCentralTransitive || !targetFrameworkInformation.CentralPackageVersions?.ContainsKey(node.Item.Key.Name) == true)
+                    {
+                        continue;
+                    }
+
+                    if (dependencyDictionary == null)
+                    {
+                        dependencyDictionary = rootNode.Item.Data.Dependencies.ToDictionary(x => x.Name, x => x, StringComparer.OrdinalIgnoreCase);
+                    }
+
+                    CentralPackageVersion centralPackageVersion = targetFrameworkInformation.CentralPackageVersions[node.Item.Key.Name];
+                    Dictionary<string, LibraryIncludeFlags> dependenciesIncludeFlags = _includeFlagGraphs[targetGraph];
+
+                    LibraryIncludeFlags suppressParent = LibraryIncludeFlags.All;
+
+                    // Centrally pinned dependencies are not directly declared but the intersection of all of the PrivateAssets of the parents that pulled it in should apply to it
+                    foreach (GraphNode<RemoteResolveResult> dependencyNode in EnumerateNodesForDependencyChecks(visitedNodes, queue, rootNode, node))
+                    {
+                        LibraryDependency dependency = dependencyDictionary[dependencyNode.Key.Name];
+                        suppressParent &= dependency.SuppressParent;
+                    }
+
+                    // If all assets are suppressed then the dependency should not be added
+                    if (suppressParent != LibraryIncludeFlags.All)
+                    {
+                        yield return new LibraryDependency()
+                        {
+                            LibraryRange = new LibraryRange(centralPackageVersion.Name, centralPackageVersion.VersionRange, LibraryDependencyTarget.Package),
+                            ReferenceType = LibraryDependencyReferenceType.Transitive,
+                            VersionCentrallyManaged = true,
+                            IncludeType = dependenciesIncludeFlags[centralPackageVersion.Name],
+                            SuppressParent = suppressParent,
+                        };
+                    }
+                }
+            }
+        }
+
+        /// <summary>
+        /// Enumerates all inner nodes of the root node which directly or transitively reference the particular graph node.
+        /// </summary>
+        /// <typeparam name="T">The type of the node.</typeparam>
+        /// <param name="visitedNodes">Reusable <see cref="HashSet{GraphNode{T}}" /> for graph traversal algorithm.</param>
+        /// <param name="queue">Reusable <see cref="Queue{GraphNode{T}}" /> for graph traversal algorithm.</param>
+        /// <param name="rootNode">The <see cref="GraphNode{TItem}" /> to know which nodes are first level inner nodes.</param>
+        /// <param name="graphNode">The <see cref="GraphNode{TItem}" /> to enumerate the parent nodes of.</param>
+        /// <returns>An <see cref="IEnumerable{T}" /> containing list of parent nodes of the specified node.</returns>
+        private static IEnumerable<GraphNode<T>> EnumerateNodesForDependencyChecks<T>(HashSet<GraphNode<T>> visitedNodes, Queue<GraphNode<T>> queue, GraphNode<T> rootNode, GraphNode<T> graphNode)
+        {
+            visitedNodes.Clear();
+            queue.Clear();
+
+            queue.Enqueue(graphNode);
+
+            while (queue.Count > 0)
+            {
+                var node = queue.Dequeue();
+                if (!visitedNodes.Add(node))
+                    continue;
+
+                if (node.Item.IsCentralTransitive)
+                {
+                    foreach (var parentNode in node.ParentNodes)
+                    {
+                        queue.Enqueue(parentNode);
+                    }
+                }
+                else
+                {
+                    if (node.OuterNode == rootNode)
+                    {
+                        // We were traversing the graph upwards and now found a parent node (an inner node of the root node).
+                        // Now we return it, so it can be used to calculate the effective value of SuppressParent for the initial node.
+                        yield return node;
+                    }
+                    else
+                    {
+                        // Go one level up
+                        queue.Enqueue(node.OuterNode);
+                    }
+                }
+            }
+        }
+
         private static void PopulatePackageFolders(IEnumerable<string> packageFolders, LockFile lockFile)
         {
             lockFile.PackageFolders.AddRange(packageFolders.Select(path => new LockFileItem(path)));
@@ -580,5 +646,49 @@ namespace NuGet.Commands
         {
             return file.StartsWith("tools/", StringComparison.OrdinalIgnoreCase);
         }
+
+        /// <summary>
+        /// An <see cref="IEqualityComparer{T}" /> that compares <see cref="LockFileTargetLibrary" /> objects by the value of the <see cref="LockFileTargetLibrary.Name" /> and <see cref="LockFileTargetLibrary.Version" /> properties.
+        /// </summary>
+        private class LockFileTargetLibraryNameAndVersionEqualityComparer : IEqualityComparer<LockFileTargetLibrary>
+        {
+            /// <summary>
+            /// Gets a static singleton for the <see cref="LockFileTargetLibraryNameAndVersionEqualityComparer" /> class.
+            /// </summary>
+            public static LockFileTargetLibraryNameAndVersionEqualityComparer Instance = new();
+
+            /// <summary>
+            /// Initializes a new instance of the <see cref="LockFileTargetLibraryNameAndVersionEqualityComparer" /> class.
+            /// </summary>
+            private LockFileTargetLibraryNameAndVersionEqualityComparer()
+            {
+            }
+
+            /// <summary>
+            /// Determines whether the specified <see cref="LockFileTargetLibrary" /> objects are equal by comparing their <see cref="LockFileTargetLibrary.Name" /> and <see cref="LockFileTargetLibrary.Version" /> properties.
+            /// </summary>
+            /// <param name="x">The first <see cref="LockFileTargetLibrary" /> to compare.</param>
+            /// <param name="y">The second <see cref="LockFileTargetLibrary" /> to compare.</param>
+            /// <returns><see langword="true" /> if the specified <see cref="LockFileTargetLibrary" /> objects' <see cref="LockFileTargetLibrary.Name" /> and <see cref="LockFileTargetLibrary.Version" /> properties are equal, otherwise <see langword="false" />.</returns>
+            public bool Equals(LockFileTargetLibrary x, LockFileTargetLibrary y)
+            {
+                return string.Equals(x.Name, y.Name, StringComparison.Ordinal) && x.Version.Equals(y.Version);
+            }
+
+            /// <summary>
+            /// Returns a hash code for the specified <see cref="LockFileTargetLibrary" /> object's <see cref="LockFileTargetLibrary.Name" /> property.
+            /// </summary>
+            /// <param name="obj">The <see cref="LockFileTargetLibrary" /> for which a hash code is to be returned.</param>
+            /// <returns>A hash code for the specified <see cref="LockFileTargetLibrary" /> object's <see cref="LockFileTargetLibrary.Name" /> and and <see cref="LockFileTargetLibrary.Version" /> properties.</returns>
+            public int GetHashCode(LockFileTargetLibrary obj)
+            {
+                var combiner = new HashCodeCombiner();
+
+                combiner.AddObject(obj.Name);
+                combiner.AddObject(obj.Version);
+
+                return combiner.CombinedHash;
+            }
+        }
     }
 }
