@@ -2,19 +2,20 @@
 // Licensed under the Apache License, Version 2.0. See License.txt in the project root for license information.
 
 using System;
+using System.Runtime.Serialization;
 
 namespace NuGet.Frameworks
 {
-#if NUGET_FRAMEWORKS_INTERNAL
-    internal
-#else
-    public
-#endif
-    class FrameworkException : Exception
+    [Serializable]
+    public class FrameworkException : Exception
     {
         public FrameworkException(string message)
             : base(message)
         {
         }
+
+        protected FrameworkException(SerializationInfo info, StreamingContext context) : base(info, context)
+        {
+        }
     }
 }
