@@ -462,7 +462,7 @@ namespace NuGet.Packaging
             }
             else
             {
-                return Guid.NewGuid().ToString("N");
+                return Guid.NewGuid().ToString("N", provider: null);
             }
         }
 
@@ -587,7 +587,7 @@ namespace NuGet.Packaging
                 .Select(framework => framework.GetShortFolderName()));
             if (frameworksMissingPlatformVersion.Any())
             {
-                throw new PackagingException(NuGetLogCode.NU1012, String.Format(CultureInfo.CurrentCulture, Strings.MissingTargetPlatformVersionsFromDependencyGroups, string.Join(", ", frameworksMissingPlatformVersion.OrderBy(str => str))));
+                throw new PackagingException(NuGetLogCode.NU1012, string.Format(CultureInfo.CurrentCulture, Strings.MissingTargetPlatformVersionsFromDependencyGroups, string.Join(", ", frameworksMissingPlatformVersion.OrderBy(str => str))));
             }
 
             if (version == null)
@@ -657,7 +657,7 @@ namespace NuGet.Packaging
         /// <param name="filePath">The file path to search for</param>
         /// <param name="packageFiles">The list of files to search within</param>
         /// <param name="filePathIncorrectCase">If the file was not found, this will be a path which almost matched but had the incorrect case</param>
-        /// <returns>An <see cref="IPackageFile"/> matching the specified path or <c>null</c></returns>
+        /// <returns>An <see cref="IPackageFile"/> matching the specified path or <see langword="null" /></returns>
         private static IPackageFile FindFileInPackage(string filePath, IEnumerable<IPackageFile> packageFiles, out string filePathIncorrectCase)
         {
             filePathIncorrectCase = null;
@@ -809,7 +809,7 @@ namespace NuGet.Packaging
                 set.Add(file);
             }
 
-            var managedCodeConventions = new ManagedCodeConventions(new RuntimeGraph());
+            var managedCodeConventions = new ManagedCodeConventions(RuntimeGraph.Empty);
             var collection = new ContentItemCollection();
             collection.Load(set.Select(path => path.Replace('\\', '/')).ToArray());
 
@@ -830,13 +830,15 @@ namespace NuGet.Packaging
             };
             var warnPaths = new HashSet<string>();
 
-            var frameworksMissingPlatformVersion = new HashSet<string>();
+            var itemsWithFrameworkMissingPlatformVersion = new HashSet<string>();
+            List<ContentItemGroup> targetedItemGroups = new();
             foreach (var pattern in frameworkPatterns)
             {
-                IEnumerable<ContentItemGroup> targetedItemGroups = ContentExtractor.GetContentForPattern(collection, pattern);
+                targetedItemGroups.Clear();
+                ContentExtractor.GetContentForPattern(collection, pattern, targetedItemGroups);
                 foreach (ContentItemGroup group in targetedItemGroups)
                 {
-                    foreach (ContentItem item in group.Items)
+                    foreach (ContentItem item in group.Items.NoAllocEnumerate())
                     {
                         var framework = (NuGetFramework)item.Properties["tfm"];
                         if (framework == null)
@@ -846,15 +848,15 @@ namespace NuGet.Packaging
 
                         if (framework.HasPlatform && framework.PlatformVersion == FrameworkConstants.EmptyVersion)
                         {
-                            frameworksMissingPlatformVersion.Add(framework.GetShortFolderName());
+                            itemsWithFrameworkMissingPlatformVersion.Add(item.Path);
                         }
                     }
                 }
             }
 
-            if (frameworksMissingPlatformVersion.Any())
+            if (itemsWithFrameworkMissingPlatformVersion.Any())
             {
-                throw new PackagingException(NuGetLogCode.NU1012, string.Format(CultureInfo.CurrentCulture, Strings.MissingTargetPlatformVersionsFromIncludedFiles, string.Join(", ", frameworksMissingPlatformVersion.OrderBy(str => str))));
+                throw new PackagingException(NuGetLogCode.NU1012, string.Format(CultureInfo.CurrentCulture, Strings.MissingTargetPlatformVersionsFromIncludedFiles, string.Join(", ", itemsWithFrameworkMissingPlatformVersion.OrderBy(str => str))));
             }
         }
 
@@ -1006,7 +1008,7 @@ namespace NuGet.Packaging
             return entry;
         }
 
-        private ZipArchiveEntry CreatePackageFileEntry(ZipArchive package, string entryName, DateTimeOffset timeOffset, CompressionLevel compressionLevel, StringBuilder warningMessage)
+        private static ZipArchiveEntry CreatePackageFileEntry(ZipArchive package, string entryName, DateTimeOffset timeOffset, CompressionLevel compressionLevel, StringBuilder warningMessage)
         {
             var entry = package.CreateEntry(entryName, compressionLevel);
 
