@@ -99,6 +99,7 @@ namespace NuGet.ProjectModel
 
                 switch (propertyName)
                 {
+#pragma warning disable CS0612 // Type or member is obsolete
                     case "authors":
                         packageSpec.Authors = ReadStringArray(jsonReader) ?? Array.Empty<string>();
                         break;
@@ -119,6 +120,7 @@ namespace NuGet.ProjectModel
                     case "copyright":
                         packageSpec.Copyright = jsonReader.ReadNextTokenAsString();
                         break;
+#pragma warning restore CS0612 // Type or member is obsolete
 
                     case "dependencies":
                         ReadDependencies(
@@ -128,14 +130,17 @@ namespace NuGet.ProjectModel
                             isGacOrFrameworkReference: false);
                         break;
 
+#pragma warning disable CS0612 // Type or member is obsolete
                     case "description":
                         packageSpec.Description = jsonReader.ReadNextTokenAsString();
                         break;
+#pragma warning restore CS0612 // Type or member is obsolete
 
                     case "frameworks":
                         ReadFrameworks(jsonReader, packageSpec);
                         break;
 
+#pragma warning disable CS0612 // Type or member is obsolete
                     case "language":
                         packageSpec.Language = jsonReader.ReadNextTokenAsString();
                         break;
@@ -148,6 +153,7 @@ namespace NuGet.ProjectModel
                         ReadPackOptions(jsonReader, packageSpec, ref isMappingsNull);
                         wasPackOptionsSet = true;
                         break;
+#pragma warning restore CS0612 // Type or member is obsolete
 
                     case "restore":
                         ReadMSBuildMetadata(jsonReader, packageSpec);
@@ -157,9 +163,11 @@ namespace NuGet.ProjectModel
                         runtimeDescriptions = ReadRuntimes(jsonReader);
                         break;
 
+#pragma warning disable CS0612 // Type or member is obsolete
                     case "scripts":
                         ReadScripts(jsonReader, packageSpec);
                         break;
+#pragma warning restore CS0612 // Type or member is obsolete
 
                     case "supports":
                         compatibilityProfiles = ReadSupports(jsonReader);
@@ -176,7 +184,9 @@ namespace NuGet.ProjectModel
                         {
                             try
                             {
+#pragma warning disable CS0612 // Type or member is obsolete
                                 packageSpec.HasVersionSnapshot = PackageSpecUtility.IsSnapshotVersion(version);
+#pragma warning restore CS0612 // Type or member is obsolete
                                 packageSpec.Version = PackageSpecUtility.SpecifySnapshot(version, snapshotValue);
                             }
                             catch (Exception ex)
@@ -191,6 +201,7 @@ namespace NuGet.ProjectModel
             packageSpec.Name = name;
             packageSpec.FilePath = name == null ? null : Path.GetFullPath(packageSpecPath);
 
+#pragma warning disable CS0612 // Type or member is obsolete
             if (!wasPackOptionsSet)
             {
                 packageSpec.Owners = Array.Empty<string>();
@@ -205,6 +216,7 @@ namespace NuGet.ProjectModel
             {
                 packageSpec.PackOptions.Mappings = null;
             }
+#pragma warning restore CS0612 // Type or member is obsolete
 
             packageSpec.RuntimeGraph = new RuntimeGraph(
                 runtimeDescriptions ?? Enumerable.Empty<RuntimeDescription>(),
@@ -232,6 +244,7 @@ namespace NuGet.ProjectModel
             return new PackageType(name, Packaging.Core.PackageType.EmptyVersion);
         }
 
+        [Obsolete]
         private static void ReadBuildOptions(JsonTextReader jsonReader, PackageSpec packageSpec)
         {
             packageSpec.BuildOptions = new BuildOptions();
@@ -356,6 +369,7 @@ namespace NuGet.ProjectModel
                     var versionCentrallyManaged = false;
                     string aliases = null;
                     string dependencyVersionValue = null;
+                    VersionRange versionOverride = null;
 
                     if (jsonReader.TokenType == JsonToken.String)
                     {
@@ -409,7 +423,23 @@ namespace NuGet.ProjectModel
                                         dependencyVersionValue = (string)jsonReader.Value;
                                     }
                                     break;
-
+                                case "versionOverride":
+                                    if (jsonReader.ReadNextToken())
+                                    {
+                                        try
+                                        {
+                                            versionOverride = VersionRange.Parse((string)jsonReader.Value);
+                                        }
+                                        catch (Exception ex)
+                                        {
+                                            throw FileFormatException.Create(
+                                                ex,
+                                                jsonReader.LineNumber,
+                                                jsonReader.LinePosition,
+                                                packageSpecPath);
+                                        }
+                                    }
+                                    break;
                                 case "versionCentrallyManaged":
                                     versionCentrallyManaged = ReadNextTokenAsBoolOrFalse(jsonReader, packageSpecPath);
                                     break;
@@ -475,7 +505,8 @@ namespace NuGet.ProjectModel
                         Aliases = aliases,
                         // The ReferenceType is not persisted to the assets file
                         // Default to LibraryDependencyReferenceType.Direct on Read
-                        ReferenceType = LibraryDependencyReferenceType.Direct
+                        ReferenceType = LibraryDependencyReferenceType.Direct,
+                        VersionOverride = versionOverride
                     };
 
                     if (noWarn != null)
@@ -488,7 +519,7 @@ namespace NuGet.ProjectModel
             });
         }
 
-        internal static void ReadCentralTransitveDependencyGroup(
+        internal static void ReadCentralTransitiveDependencyGroup(
             JsonTextReader jsonReader,
             IList<LibraryDependency> results,
             string packageSpecPath)
@@ -777,6 +808,7 @@ namespace NuGet.ProjectModel
                     {
                         throw FileFormatException.Create(
                             string.Format(
+                                CultureInfo.CurrentCulture,
                                 Strings.Log_InvalidImportFramework,
                                 import,
                                 PackageSpec.PackageSpecFileName),
@@ -870,6 +902,8 @@ namespace NuGet.ProjectModel
         private static void ReadMSBuildMetadata(JsonTextReader jsonReader, PackageSpec packageSpec)
         {
             var centralPackageVersionsManagementEnabled = false;
+            var centralPackageVersionOverrideDisabled = false;
+            var CentralPackageTransitivePinningEnabled = false;
             List<string> configFilePaths = null;
             var crossTargeting = false;
             List<string> fallbackFolders = null;
@@ -891,6 +925,7 @@ namespace NuGet.ProjectModel
             List<ProjectRestoreMetadataFrameworkInfo> targetFrameworks = null;
             var validateRuntimeAssets = false;
             WarningProperties warningProperties = null;
+            RestoreAuditProperties auditProperties = null;
 
             jsonReader.ReadObject(propertyName =>
             {
@@ -900,6 +935,14 @@ namespace NuGet.ProjectModel
                         centralPackageVersionsManagementEnabled = ReadNextTokenAsBoolOrFalse(jsonReader, packageSpec.FilePath);
                         break;
 
+                    case "centralPackageVersionOverrideDisabled":
+                        centralPackageVersionOverrideDisabled = ReadNextTokenAsBoolOrFalse(jsonReader, packageSpec.FilePath);
+                        break;
+
+                    case "CentralPackageTransitivePinningEnabled":
+                        CentralPackageTransitivePinningEnabled = ReadNextTokenAsBoolOrFalse(jsonReader, packageSpec.FilePath);
+                        break;
+
                     case "configFilePaths":
                         configFilePaths = jsonReader.ReadStringArrayAsList();
                         break;
@@ -997,6 +1040,34 @@ namespace NuGet.ProjectModel
                         restoreLockProperties = new RestoreLockProperties(restorePackagesWithLockFile, nuGetLockFilePath, restoreLockedMode);
                         break;
 
+                    case "restoreAuditProperties":
+                        string enableAudit = null, auditLevel = null, auditMode = null;
+                        jsonReader.ReadObject(auditPropertyName =>
+                        {
+
+                            switch (auditPropertyName)
+                            {
+                                case "enableAudit":
+                                    enableAudit = jsonReader.ReadNextTokenAsString();
+                                    break;
+
+                                case "auditLevel":
+                                    auditLevel = jsonReader.ReadNextTokenAsString();
+                                    break;
+
+                                case "auditMode":
+                                    auditMode = jsonReader.ReadNextTokenAsString();
+                                    break;
+                            }
+                        });
+                        auditProperties = new RestoreAuditProperties()
+                        {
+                            EnableAudit = enableAudit,
+                            AuditLevel = auditLevel,
+                            AuditMode = auditMode,
+                        };
+                        break;
+
                     case "skipContentFileWrite":
                         skipContentFileWrite = ReadNextTokenAsBoolOrFalse(jsonReader, packageSpec.FilePath);
                         break;
@@ -1018,6 +1089,7 @@ namespace NuGet.ProjectModel
                         var allWarningsAsErrors = false;
                         var noWarn = new HashSet<NuGetLogCode>();
                         var warnAsError = new HashSet<NuGetLogCode>();
+                        var warningsNotAsErrors = new HashSet<NuGetLogCode>();
 
                         jsonReader.ReadObject(warningPropertiesPropertyName =>
                         {
@@ -1034,10 +1106,14 @@ namespace NuGet.ProjectModel
                                 case "warnAsError":
                                     ReadNuGetLogCodes(jsonReader, warnAsError);
                                     break;
+
+                                case "warnNotAsError":
+                                    ReadNuGetLogCodes(jsonReader, warningsNotAsErrors);
+                                    break;
                             }
                         });
 
-                        warningProperties = new WarningProperties(warnAsError, noWarn, allWarningsAsErrors);
+                        warningProperties = new WarningProperties(warnAsError, noWarn, allWarningsAsErrors, warningsNotAsErrors);
                         break;
                 }
             });
@@ -1055,6 +1131,9 @@ namespace NuGet.ProjectModel
             }
 
             msbuildMetadata.CentralPackageVersionsEnabled = centralPackageVersionsManagementEnabled;
+            msbuildMetadata.CentralPackageVersionOverrideDisabled = centralPackageVersionOverrideDisabled;
+            msbuildMetadata.CentralPackageTransitivePinningEnabled = CentralPackageTransitivePinningEnabled;
+            msbuildMetadata.RestoreAuditProperties = auditProperties;
 
             if (configFilePaths != null)
             {
@@ -1226,10 +1305,12 @@ namespace NuGet.ProjectModel
                             throw new InvalidCastException();
                     }
 
+#pragma warning disable CS0612 // Type or member is obsolete
                     if (packageTypes != null)
                     {
                         packageSpec.PackOptions.PackageType = packageTypes;
                     }
+#pragma warning restore CS0612 // Type or member is obsolete
                 }
             }
             catch (Exception)
@@ -1245,6 +1326,7 @@ namespace NuGet.ProjectModel
             }
         }
 
+        [Obsolete]
         private static void ReadPackInclude(JsonTextReader jsonReader, PackageSpec packageSpec)
         {
             jsonReader.ReadObject(propertyName =>
@@ -1255,6 +1337,7 @@ namespace NuGet.ProjectModel
             });
         }
 
+        [Obsolete]
         private static void ReadPackOptions(JsonTextReader jsonReader, PackageSpec packageSpec, ref bool isMappingsNull)
         {
             var wasMappingsRead = false;
@@ -1318,6 +1401,7 @@ namespace NuGet.ProjectModel
             isMappingsNull = isPackOptionsValueAnObject && !wasMappingsRead;
         }
 
+        [Obsolete]
         private static bool ReadPackOptionsFiles(PackageSpec packageSpec, JsonTextReader jsonReader, bool wasMappingsRead)
         {
             IReadOnlyList<string> excludeFiles = null;
@@ -1376,7 +1460,7 @@ namespace NuGet.ProjectModel
 
             jsonReader.ReadObject(propertyName =>
             {
-                dependencies = dependencies ?? new List<RuntimePackageDependency>();
+                dependencies ??= new List<RuntimePackageDependency>();
 
                 var dependency = new RuntimePackageDependency(propertyName, VersionRange.Parse(jsonReader.ReadNextTokenAsString()));
 
@@ -1385,7 +1469,7 @@ namespace NuGet.ProjectModel
 
             return new RuntimeDependencySet(
                 dependencySetName,
-                dependencies ?? Enumerable.Empty<RuntimePackageDependency>());
+                dependencies);
         }
 
         private static RuntimeDescription ReadRuntimeDescription(JsonTextReader jsonReader, string runtimeName)
@@ -1401,7 +1485,7 @@ namespace NuGet.ProjectModel
                 }
                 else
                 {
-                    additionalDependencies = additionalDependencies ?? new List<RuntimeDependencySet>();
+                    additionalDependencies ??= new List<RuntimeDependencySet>();
 
                     RuntimeDependencySet dependency = ReadRuntimeDependencySet(jsonReader, propertyName);
 
@@ -1411,8 +1495,8 @@ namespace NuGet.ProjectModel
 
             return new RuntimeDescription(
                 runtimeName,
-                inheritedRuntimes ?? Enumerable.Empty<string>(),
-                additionalDependencies ?? Enumerable.Empty<RuntimeDependencySet>());
+                inheritedRuntimes,
+                additionalDependencies);
         }
 
         private static List<RuntimeDescription> ReadRuntimes(JsonTextReader jsonReader)
@@ -1429,6 +1513,7 @@ namespace NuGet.ProjectModel
             return runtimeDescriptions;
         }
 
+        [Obsolete]
         private static void ReadScripts(JsonTextReader jsonReader, PackageSpec packageSpec)
         {
             jsonReader.ReadObject(propertyName =>
@@ -1453,7 +1538,7 @@ namespace NuGet.ProjectModel
                     else
                     {
                         throw FileFormatException.Create(
-                            string.Format("The value of a script in '{0}' can only be a string or an array of strings", PackageSpec.PackageSpecFileName),
+                            string.Format(CultureInfo.CurrentCulture, "The value of a script in '{0}' can only be a string or an array of strings", PackageSpec.PackageSpecFileName),
                             jsonReader.LineNumber,
                             jsonReader.LinePosition,
                             packageSpec.FilePath);
