@@ -29,13 +29,7 @@ namespace NuGet.PackageManagement.VisualStudio.Test
             // dependencies and data
             _metadataResource = Mock.Of<PackageMetadataResource>();
 
-            var provider = Mock.Of<INuGetResourceProvider>();
-            Mock.Get(provider)
-                .Setup(x => x.TryCreate(It.IsAny<SourceRepository>(), It.IsAny<CancellationToken>()))
-                .Returns(() => Task.FromResult(Tuple.Create(true, (INuGetResource)_metadataResource)));
-            Mock.Get(provider)
-                .Setup(x => x.ResourceType)
-                .Returns(typeof(PackageMetadataResource));
+            INuGetResourceProvider provider = FeedTestUtils.CreateTestResourceProvider(_metadataResource);
 
             var packageSource = new Configuration.PackageSource("http://fake-source");
             var source = new SourceRepository(packageSource, new[] { provider });
