@@ -137,11 +137,13 @@ namespace NuGet.ProjectModel.Test
                 generatePathProperty: false,
                 versionCentrallyManaged: false,
                 LibraryDependencyReferenceType.Direct,
-                aliases: "stuffff");
+                aliases: "stuffff",
+                versionOverride: null);
 
             return dependency;
         }
 
+        [Obsolete]
         private IncludeExcludeFiles CreateIncludeExcludeFiles()
         {
             var files = new IncludeExcludeFiles();
@@ -152,10 +154,11 @@ namespace NuGet.ProjectModel.Test
             return files;
         }
 
+        [Obsolete]
         private PackOptions CreatePackOptions()
         {
             var originalPackOptions = new PackOptions();
-            originalPackOptions.PackageType = new List<PackageType>() { new PackageType("PackageA", new System.Version("1.0.0")) };
+            originalPackOptions.PackageType = new List<PackageType>() { new PackageType("PackageA", new Version("1.0.0")) };
             originalPackOptions.IncludeExcludeFiles = CreateIncludeExcludeFiles();
             return originalPackOptions;
         }
@@ -168,7 +171,8 @@ namespace NuGet.ProjectModel.Test
             packageSpec.FilePath = "FilePath";
             packageSpec.Name = "Name";
             packageSpec.Title = "Title";
-            packageSpec.Version = new Versioning.NuGetVersion("1.0.0");
+            packageSpec.Version = new NuGetVersion("1.0.0");
+#pragma warning disable CS0612 // Type or member is obsolete
             packageSpec.HasVersionSnapshot = true;
             packageSpec.Description = "Description";
             packageSpec.Summary = "Summary";
@@ -184,7 +188,6 @@ namespace NuGet.ProjectModel.Test
             packageSpec.Tags = new string[] { "Tags" };
             packageSpec.BuildOptions = CreateBuildOptions();
             packageSpec.ContentFiles = new List<string>() { "contentFile1", "contentFile2" };
-            packageSpec.Dependencies = new List<LibraryDependency>() { CreateLibraryDependency(), CreateLibraryDependency() };
 
             packageSpec.Scripts.Add(Guid.NewGuid().ToString(), new List<string>() { Guid.NewGuid().ToString() });
             packageSpec.Scripts.Add(Guid.NewGuid().ToString(), new List<string>() { Guid.NewGuid().ToString() });
@@ -192,7 +195,9 @@ namespace NuGet.ProjectModel.Test
             packageSpec.PackInclude.Add(Guid.NewGuid().ToString(), Guid.NewGuid().ToString());
 
             packageSpec.PackOptions = CreatePackOptions();
+#pragma warning restore CS0612 // Type or member is obsolete
 
+            packageSpec.Dependencies = new List<LibraryDependency>() { CreateLibraryDependency(), CreateLibraryDependency() };
             packageSpec.RuntimeGraph = CreateRuntimeGraph();
             packageSpec.RestoreSettings = CreateProjectRestoreSettings();
             return packageSpec;
@@ -257,6 +262,7 @@ namespace NuGet.ProjectModel.Test
 
         public class PackageSpecModify
         {
+            [Obsolete]
             public static void ModifyAuthors(PackageSpec packageSpec)
             {
                 packageSpec.Authors[0] = "NewAuthor";
@@ -282,21 +288,25 @@ namespace NuGet.ProjectModel.Test
                 packageSpec.Version = new Versioning.NuGetVersion("2.0.0");
             }
 
+            [Obsolete]
             public static void ModifyOwners(PackageSpec packageSpec)
             {
                 packageSpec.Owners[0] = "BetterOwner";
             }
 
+            [Obsolete]
             public static void ModifyTags(PackageSpec packageSpec)
             {
                 packageSpec.Tags[0] = "better tag!";
             }
 
+            [Obsolete]
             public static void ModifyBuildOptions(PackageSpec packageSpec)
             {
                 packageSpec.BuildOptions.OutputName = Guid.NewGuid().ToString();
             }
 
+            [Obsolete]
             public static void ModifyContentFiles(PackageSpec packageSpec)
             {
                 packageSpec.ContentFiles.Add("New fnacy content file");
@@ -307,11 +317,13 @@ namespace NuGet.ProjectModel.Test
                 packageSpec.Dependencies.Add(CreateLibraryDependency());
             }
 
+            [Obsolete]
             public static void ModifyScriptsAdd(PackageSpec packageSpec)
             {
                 packageSpec.Scripts.Add(Guid.NewGuid().ToString(), new List<string>() { Guid.NewGuid().ToString() });
             }
 
+            [Obsolete]
             public static void ModifyScriptsEdit(PackageSpec packageSpec)
             {
                 var enumerator = packageSpec.Scripts.Keys.GetEnumerator();
@@ -320,11 +332,13 @@ namespace NuGet.ProjectModel.Test
                 ((List<string>)packageSpec.Scripts[key]).Add(Guid.NewGuid().ToString());
             }
 
+            [Obsolete]
             public static void ModifyPackInclude(PackageSpec packageSpec)
             {
                 packageSpec.PackInclude.Add(Guid.NewGuid().ToString(), Guid.NewGuid().ToString());
             }
 
+            [Obsolete]
             public static void ModifyPackOptions(PackageSpec packageSpec)
             {
                 ((List<PackageType>)packageSpec.PackOptions.PackageType).Add(PackageType.DotnetCliTool);
@@ -348,7 +362,8 @@ namespace NuGet.ProjectModel.Test
             var allWarningsAsErrors = true;
             var noWarn = new HashSet<NuGetLogCode>() { NuGetLogCode.NU1000, NuGetLogCode.NU1500 };
             var warningsAsErrors = new HashSet<NuGetLogCode>() { NuGetLogCode.NU1001, NuGetLogCode.NU1501 };
-            var warningProperties = new WarningProperties(allWarningsAsErrors: allWarningsAsErrors, warningsAsErrors: warningsAsErrors, noWarn: noWarn);
+            var warningsNotAsErrors = new HashSet<NuGetLogCode>() { NuGetLogCode.NU1005 };
+            var warningProperties = new WarningProperties(allWarningsAsErrors: allWarningsAsErrors, warningsAsErrors: warningsAsErrors, noWarn: noWarn, warningsNotAsErrors: warningsNotAsErrors);
 
             var originalProjectRestoreMetadata = new ProjectRestoreMetadata();
             originalProjectRestoreMetadata.ProjectStyle = ProjectStyle.PackageReference;
@@ -613,7 +628,8 @@ namespace NuGet.ProjectModel.Test
                 generatePathProperty: false,
                 versionCentrallyManaged: false,
                 LibraryDependencyReferenceType.Direct,
-                aliases: "stuff");
+                aliases: "stuff",
+                versionOverride: null);
             var imports = NuGetFramework.Parse("net45"); // This makes no sense in the context of fallback, just for testing :)
 
             var originalTargetFrameworkInformation = new TargetFrameworkInformation();
@@ -719,7 +735,8 @@ namespace NuGet.ProjectModel.Test
             var allWarningsAsErrors = false;
             var noWarn = new HashSet<NuGetLogCode>() { NuGetLogCode.NU1000, NuGetLogCode.NU1500 };
             var warningsAsErrors = new HashSet<NuGetLogCode>() { NuGetLogCode.NU1001, NuGetLogCode.NU1501 };
-            var originalWarningProperties = new WarningProperties(allWarningsAsErrors: allWarningsAsErrors, warningsAsErrors: warningsAsErrors, noWarn: noWarn);
+            var warningsNotAsErrors = new HashSet<NuGetLogCode>() { NuGetLogCode.NU1005 };
+            var originalWarningProperties = new WarningProperties(allWarningsAsErrors: allWarningsAsErrors, warningsAsErrors: warningsAsErrors, noWarn: noWarn, warningsNotAsErrors: warningsNotAsErrors);
 
             //Act
             var clone = originalWarningProperties.Clone();
@@ -743,6 +760,7 @@ namespace NuGet.ProjectModel.Test
         }
 
         [Fact]
+        [Obsolete]
         public void FrameworkRuntimePairCloneTest()
         {
             //Setup
@@ -750,8 +768,7 @@ namespace NuGet.ProjectModel.Test
             //Act
             var clone = frp.Clone();
             //Assert
-            Assert.Equal(frp, clone);
-            Assert.False(object.ReferenceEquals(frp, clone));
+            Assert.Same(frp, clone);
         }
 
         private static CompatibilityProfile CreateCompatibilityProfile(string name, string tfm = "net461")
@@ -769,7 +786,7 @@ namespace NuGet.ProjectModel.Test
             //Assert
             Assert.Equal(compat, clone);
             Assert.False(object.ReferenceEquals(compat, clone));
-            Assert.False(object.ReferenceEquals(compat.RestoreContexts[0], clone.RestoreContexts[0]));
+            Assert.Same(compat.RestoreContexts[0], clone.RestoreContexts[0]); // FRP is immutable so the instance is reused
 
             // Act - Change the list of compat
             compat.RestoreContexts.Add(CreateFrameworkRuntimePair(rid: "win10-x64"));
@@ -780,6 +797,7 @@ namespace NuGet.ProjectModel.Test
             Assert.Equal(2, clone.RestoreContexts.Count);
         }
 
+        [Obsolete]
         [Theory]
         [InlineData(true)]
         [InlineData(false)]
