@@ -575,9 +575,9 @@ namespace NuGet.Protocol.Tests
             TestEnvironmentVariableReader testEnvironmentVariableReader = new TestEnvironmentVariableReader(
             new Dictionary<string, string>()
             {
-                            { "NUGET_ENABLE_EXPERIMENTAL_HTTP_RETRY" , "true"},
-                            { "NUGET_EXPERIMENTAL_MAX_NETWORK_TRY_COUNT" , $"{testTryCount}"},
-                            { "NUGET_EXPERIMENTAL_NETWORK_RETRY_DELAY_MILLISECONDS" , "0"}
+                [EnhancedHttpRetryHelper.IsEnabledEnvironmentVariableName] = bool.TrueString,
+                [EnhancedHttpRetryHelper.RetryCountEnvironmentVariableName] = testTryCount.ToString(),
+                [EnhancedHttpRetryHelper.DelayInMillisecondsEnvironmentVariableName] = "0"
             });
             using (var test = await HttpFileSystemBasedFindPackageByIdResourceTest.CreateAsync(testEnvironmentVariableReader))
             {
@@ -596,7 +596,7 @@ namespace NuGet.Protocol.Tests
         private static HttpSource CreateDummyHttpSource()
         {
             var packageSource = new PackageSource("https://unit.test");
-            Task<HttpHandlerResource> messageHandlerFactory() => Task.FromResult<HttpHandlerResource>(null);
+            Task<HttpHandlerResource> messageHandlerFactory() => TaskResult.Null<HttpHandlerResource>();
 
             return new HttpSource(packageSource, messageHandlerFactory, Mock.Of<IThrottle>());
         }
