@@ -1,16 +1,12 @@
-// Copyright (c) .NET Foundation and contributors. All rights reserved.
-// Licensed under the MIT license. See LICENSE file in the project root for full license information.
+// Licensed to the .NET Foundation under one or more agreements.
+// The .NET Foundation licenses this file to you under the MIT license.
 
 using Microsoft.Build.Construction;
-using Microsoft.Build.Evaluation;
 using Microsoft.Build.Exceptions;
 using Microsoft.Build.Execution;
 using Microsoft.DotNet.Cli.Sln.Internal;
 using Microsoft.DotNet.Cli.Utils;
-using System;
 using System.Collections.Generic;
-using System.IO;
-using System.Linq;
 
 namespace Microsoft.DotNet.Tools.Common
 {
@@ -75,17 +71,17 @@ namespace Microsoft.DotNet.Tools.Common
                 // section comes first we need to add it first. This doesn't affect correctness but does
                 // stop VS from re-ordering things later on. Since we are keeping the SlnFile class low-level
                 // it shouldn't care about the VS implementation details. That's why we handle this here.
-                slnFile.AddDefaultBuildConfigurations();
-
-                slnFile.MapSolutionConfigurationsToProject(
-                    projectInstance,
-                    slnFile.ProjectConfigurationsSection.GetOrCreatePropertySet(slnProject.Id));
-
-                if (solutionFolders != null)
+                if (AreBuildConfigurationsApplicable(slnProject.TypeGuid))
                 {
-                    slnFile.AddSolutionFolders(slnProject, solutionFolders);
+                    slnFile.AddDefaultBuildConfigurations();
+
+                    slnFile.MapSolutionConfigurationsToProject(
+                        projectInstance,
+                        slnFile.ProjectConfigurationsSection.GetOrCreatePropertySet(slnProject.Id));
                 }
 
+                SetupSolutionFolders(slnFile, solutionFolders, relativeProjectPath, slnProject);
+
                 slnFile.Projects.Add(slnProject);
 
                 Reporter.Output.WriteLine(
@@ -93,6 +89,42 @@ namespace Microsoft.DotNet.Tools.Common
             }
         }
 
+        private static bool AreBuildConfigurationsApplicable(string projectTypeGuid)
+        {
+            return !projectTypeGuid.Equals(ProjectTypeGuids.SharedProjectGuid, StringComparison.OrdinalIgnoreCase);
+        }
+
+        private static void SetupSolutionFolders(SlnFile slnFile, IList<string> solutionFolders, string relativeProjectPath, SlnProject slnProject)
+        {
+            if (solutionFolders != null)
+            {
+                if (solutionFolders.Any())
+                {
+                    // Before adding a solution folder, check if the name conflicts with any existing projects in the solution
+                    var duplicateProjects = slnFile.Projects.Where(p => solutionFolders.Contains(p.Name)
+                                                                    && p.TypeGuid != ProjectTypeGuids.SolutionFolderGuid).ToList();
+                    foreach (SlnProject duplicateProject in duplicateProjects)
+                    {
+                        slnFile.AddSolutionFolders(duplicateProject, new List<string>() { Path.GetDirectoryName(duplicateProject.FilePath) });
+                    }
+                }
+                else
+                {
+                    // If a project and solution folder have the same name, add it's own folder as a solution folder
+                    // eg. foo\extensions.csproj and extensions\library\library.csproj would have a project and solution folder with conflicting names
+                    var duplicateProject = slnFile.Projects.Where(p => string.Equals(p.Name, slnProject.Name, StringComparison.OrdinalIgnoreCase)
+                                                                   && p.TypeGuid == ProjectTypeGuids.SolutionFolderGuid).FirstOrDefault();
+                    if (duplicateProject != null)
+                    {
+                        // Try making a new folder for the project to put it under so we can still add it despite there being one with the same name already in the parent folder
+                        slnFile.AddSolutionFolders(slnProject, new List<string>() { Path.GetDirectoryName(relativeProjectPath) });
+                    }
+                }
+                // Even if we added a solution folder above for a duplicate, we still need to add the expected folder for the current project
+                slnFile.AddSolutionFolders(slnProject, solutionFolders);
+            }
+        }
+
         private static void AddDefaultBuildConfigurations(this SlnFile slnFile)
         {
             var configurationsSection = slnFile.SolutionConfigurationsSection;
@@ -190,7 +222,7 @@ namespace Microsoft.DotNet.Tools.Common
             Dictionary<string, string> projectPlatforms,
             string defaultProjectPlatform)
         {
-            var pair = solutionConfigKey.Split(new char[] {'|'}, 2);
+            var pair = solutionConfigKey.Split(new char[] { '|' }, 2);
             if (pair.Length != 2)
             {
                 return null;
@@ -222,6 +254,11 @@ namespace Microsoft.DotNet.Tools.Common
 
                 var pathToGuidMap = slnFile.GetSolutionFolderPaths(nestedProjectsSection.Properties);
 
+                if (slnFile.HasSolutionFolder(nestedProjectsSection.Properties, slnProject))
+                {
+                    return;
+                }
+
                 string parentDirGuid = null;
                 var solutionFolderHierarchy = string.Empty;
                 foreach (var dir in solutionFolders)
@@ -233,6 +270,12 @@ namespace Microsoft.DotNet.Tools.Common
                     }
                     else
                     {
+
+                        if(HasDuplicateNameForSameValueOfNestedProjects(nestedProjectsSection, dir, parentDirGuid, slnFile.Projects))
+                        {
+                            throw new GracefulException(CommonLocalizableStrings.SolutionFolderAlreadyContainsProject, slnFile.FullPath, slnProject.Name, slnFile.Projects.FirstOrDefault(p => p.Id == parentDirGuid).Name);
+                        }
+
                         var solutionFolder = new SlnProject
                         {
                             Id = Guid.NewGuid().ToString("B").ToUpper(),
@@ -250,11 +293,31 @@ namespace Microsoft.DotNet.Tools.Common
                         parentDirGuid = solutionFolder.Id;
                     }
                 }
-
+                if (HasDuplicateNameForSameValueOfNestedProjects(nestedProjectsSection, slnProject.Name, parentDirGuid, slnFile.Projects))
+                {
+                    throw new GracefulException(CommonLocalizableStrings.SolutionFolderAlreadyContainsProject, slnFile.FullPath, slnProject.Name, slnFile.Projects.FirstOrDefault(p => p.Id == parentDirGuid).Name);
+                }
                 nestedProjectsSection.Properties[slnProject.Id] = parentDirGuid;
             }
         }
 
+        private static bool HasDuplicateNameForSameValueOfNestedProjects(SlnSection nestedProjectsSection, string name, string value, IList<SlnProject> projects)
+        {
+            foreach (var property in nestedProjectsSection.Properties)
+            {
+                if (property.Value == value)
+                {
+                    var existingProject = projects.FirstOrDefault(p => p.Id == property.Key);
+
+                    if (existingProject != null && existingProject.Name == name)
+                    {
+                        return true;
+                    }
+                }
+            }
+            return false;
+        }
+
         private static IDictionary<string, string> GetSolutionFolderPaths(
             this SlnFile slnFile,
             SlnPropertySet nestedProjects)
@@ -269,7 +332,9 @@ namespace Microsoft.DotNet.Tools.Common
                 while (nestedProjects.ContainsKey(id))
                 {
                     id = nestedProjects[id];
-                    var parentSlnProject = solutionFolderProjects.Where(p => p.Id == id).Single();
+                    var parentSlnProject = solutionFolderProjects.Where(p => p.Id == id).SingleOrDefault();
+                    if (parentSlnProject == null) // see: https://github.com/dotnet/sdk/pull/28811
+                        throw new GracefulException(CommonLocalizableStrings.CorruptSolutionProjectFolderStructure, slnFile.FullPath, id);
                     path = Path.Combine(parentSlnProject.FilePath, path);
                 }
 
@@ -279,6 +344,14 @@ namespace Microsoft.DotNet.Tools.Common
             return solutionFolderPaths;
         }
 
+        private static bool HasSolutionFolder(
+            this SlnFile slnFile,
+            SlnPropertySet properties,
+            SlnProject slnProject)
+        {
+            return properties.ContainsKey(slnProject.Id);
+        }
+
         public static bool RemoveProject(this SlnFile slnFile, string projectPath)
         {
             if (string.IsNullOrEmpty(projectPath))
@@ -417,8 +490,9 @@ namespace Microsoft.DotNet.Tools.Common
         {
             var solutionFoldersInUse = new HashSet<string>();
 
-            var nonSolutionFolderProjects = slnFile.Projects.GetProjectsNotOfType(
-                ProjectTypeGuids.SolutionFolderGuid);
+            IEnumerable<SlnProject> nonSolutionFolderProjects;
+            nonSolutionFolderProjects = slnFile.Projects.GetProjectsNotOfType(
+                 ProjectTypeGuids.SolutionFolderGuid);
 
             foreach (var nonSolutionFolderProject in nonSolutionFolderProjects)
             {
