@@ -1,20 +1,7 @@
-// Copyright (c) .NET Foundation and contributors. All rights reserved.
-// Licensed under the MIT license. See LICENSE file in the project root for full license information.
+// Licensed to the .NET Foundation under one or more agreements.
+// The .NET Foundation licenses this file to you under the MIT license.
 
-using System;
-using System.IO;
-using System.Linq;
-using System.Runtime.InteropServices;
-using FluentAssertions;
 using Microsoft.Extensions.DependencyModel;
-using Microsoft.NET.TestFramework;
-using Microsoft.NET.TestFramework.Assertions;
-using Microsoft.NET.TestFramework.Commands;
-using Xunit;
-using System.Xml.Linq;
-using Xunit.Abstractions;
-using System.Collections.Generic;
-using Microsoft.NET.TestFramework.ProjectConstruction;
 
 namespace Microsoft.NET.Publish.Tests
 {
@@ -49,7 +36,7 @@ namespace Microsoft.NET.Publish.Tests
                 Name = "TestApp",
                 IsExe = true,
                 TargetFrameworks = appTargetFramework,
-                RuntimeIdentifier = "win7-x86"
+                RuntimeIdentifier = $"{ToolsetInfo.LatestWinRuntimeIdentifier}-x86"
             };
 
             testProject.AdditionalProperties["PreserveCompilationContext"] = "true";
@@ -60,7 +47,7 @@ namespace Microsoft.NET.Publish.Tests
             }
 
             testProject.ReferencedProjects.Add(testLibraryProject);
-            testProject.PackageReferences.Add(new TestPackageReference("Newtonsoft.Json", "9.0.1"));
+            testProject.PackageReferences.Add(new TestPackageReference("Newtonsoft.Json", ToolsetInfo.GetNewtonsoftJsonPackageVersion()));
             testProject.PackageReferences.Add(new TestPackageReference("System.Data.SqlClient", "4.4.3"));
 
             var testAsset = _testAssetsManager.CreateTestProject(testProject, identifier: appTargetFramework + withoutCopyingRefs);
@@ -77,7 +64,7 @@ namespace Microsoft.NET.Publish.Tests
                 .Should()
                 .Pass();
 
-            var publishDirectory = publishCommand.GetOutputDirectory(appTargetFramework, runtimeIdentifier: "win7-x86");
+            var publishDirectory = publishCommand.GetOutputDirectory(appTargetFramework, runtimeIdentifier: $"{ToolsetInfo.LatestWinRuntimeIdentifier}-x86");
 
             publishDirectory.Should().HaveFiles(new[] {
                 appTargetFramework == "net46" ? "TestApp.exe" : "TestApp.dll",
@@ -197,7 +184,7 @@ namespace Microsoft.NET.Publish.Tests
             File.WriteAllLines(manifestFile, new[]
             {
                 "<StoreArtifacts>",
-                @"  <Package Id=""Newtonsoft.Json"" Version=""9.0.1"" />",
+                $@"  <Package Id=""Newtonsoft.Json"" Version=""{ToolsetInfo.GetNewtonsoftJsonPackageVersion()}"" />",
                 @"  <Package Id=""System.Data.SqlClient"" Version=""4.3.0"" />",
                 "</StoreArtifacts>",
             });
@@ -210,7 +197,7 @@ namespace Microsoft.NET.Publish.Tests
                 .Should()
                 .Pass();
 
-            var publishDirectory = publishCommand.GetOutputDirectory(targetFramework, runtimeIdentifier: "win7-x86");
+            var publishDirectory = publishCommand.GetOutputDirectory(targetFramework, runtimeIdentifier: $"{ToolsetInfo.LatestWinRuntimeIdentifier}-x86");
 
             publishDirectory.Should().HaveFiles(new[] {
                 "TestApp.dll",
@@ -335,6 +322,9 @@ System.Collections.Immutable.dll
 System.ComponentModel.dll
 System.ComponentModel.Annotations.dll
 System.Console.dll
+System.Collections.NonGeneric.dll
+System.ComponentModel.Primitives.dll
+System.ComponentModel.TypeConverter.dll
 System.Data.Common.dll
 System.Data.SqlClient.dll
 System.Diagnostics.Debug.dll
@@ -382,6 +372,7 @@ System.Runtime.InteropServices.dll
 System.Runtime.InteropServices.RuntimeInformation.dll
 System.Runtime.Numerics.dll
 System.Runtime.Serialization.Primitives.dll
+System.Runtime.Serialization.Formatters.dll
 System.Security.Cryptography.Algorithms.dll
 System.Security.Cryptography.Encoding.dll
 System.Security.Cryptography.OpenSsl.dll
@@ -401,6 +392,7 @@ System.Threading.ThreadPool.dll
 System.Threading.Timer.dll
 System.Xml.ReaderWriter.dll
 System.Xml.XDocument.dll
+System.Xml.XmlDocument.dll
 TestLibrary.dll"
             },
             {
