@@ -1,14 +1,5 @@
-// Copyright (c) .NET Foundation and contributors. All rights reserved.
-// Licensed under the MIT license. See LICENSE file in the project root for full license information.
-
-using System;
-using FluentAssertions;
-using Microsoft.DotNet.Tools.Test.Utilities;
-using Microsoft.NET.TestFramework;
-using Microsoft.NET.TestFramework.Assertions;
-using Microsoft.NET.TestFramework.Commands;
-using Xunit;
-using Xunit.Abstractions;
+// Licensed to the .NET Foundation under one or more agreements.
+// The .NET Foundation licenses this file to you under the MIT license.
 
 namespace Microsoft.DotNet.Cli.Install.Tests
 {
@@ -44,7 +35,7 @@ namespace Microsoft.DotNet.Cli.Install.Tests
             result
                 .StdOut
                 .Should()
-                .ContainVisuallySameFragmentIfNotLocalized("Restoring");
+                .BeEmpty();
         }
     }
 }
