@@ -1,11 +1,11 @@
-// Copyright (c) .NET Foundation and contributors. All rights reserved.
-// Licensed under the MIT license. See LICENSE file in the project root for full license information.
+// Licensed to the .NET Foundation under one or more agreements.
+// The .NET Foundation licenses this file to you under the MIT license.
 
-using System;
-using System.Collections.Generic;
+using System.CommandLine;
 using System.CommandLine.Parsing;
 using System.IO;
 using FluentAssertions;
+using System.Runtime.CompilerServices;
 using Microsoft.DotNet.Cli;
 using Microsoft.DotNet.Cli.Utils;
 using Microsoft.DotNet.ToolManifest;
@@ -14,20 +14,22 @@ using Microsoft.DotNet.Tools.Tests.ComponentMocks;
 using Microsoft.DotNet.Tools.Tool.Restore;
 using Microsoft.Extensions.DependencyModel.Tests;
 using Microsoft.Extensions.EnvironmentAbstractions;
-using Microsoft.NET.TestFramework.Utilities;
 using NuGet.Frameworks;
 using NuGet.Versioning;
-using Xunit;
 using LocalizableStrings = Microsoft.DotNet.Tools.Tool.Restore.LocalizableStrings;
 using Parser = Microsoft.DotNet.Cli.Parser;
+using Microsoft.DotNet.Cli.ToolPackage;
+using System.Reflection;
+using System.Text.Json;
 
 namespace Microsoft.DotNet.Tests.Commands.Tool
 {
-    public class ToolRestoreCommandTests
+    public class ToolRestoreCommandTests: SdkTest
     {
         private readonly IFileSystem _fileSystem;
         private readonly IToolPackageStore _toolPackageStore;
-        private readonly ToolPackageInstallerMock _toolPackageInstallerMock;
+        private readonly ToolPackageDownloaderMock _toolPackageDownloaderMock;
+        private readonly ToolPackageDownloader _toolPackageDownloader;
         private readonly ParseResult _parseResult;
         private readonly BufferedReporter _reporter;
         private readonly string _temporaryDirectory;
@@ -49,7 +51,7 @@ namespace Microsoft.DotNet.Tests.Commands.Tool
 
         private int _installCalledCount = 0;
 
-        public ToolRestoreCommandTests()
+        public ToolRestoreCommandTests(ITestOutputHelper log): base(log)
         {
             _packageVersionA = NuGetVersion.Parse("1.0.4");
             _packageVersionWithCommandNameCollisionWithA = NuGetVersion.Parse("1.0.9");
@@ -63,41 +65,42 @@ namespace Microsoft.DotNet.Tests.Commands.Tool
             ToolPackageStoreMock toolPackageStoreMock =
                 new ToolPackageStoreMock(new DirectoryPath(_pathToPlacePackages), _fileSystem);
             _toolPackageStore = toolPackageStoreMock;
-            _toolPackageInstallerMock = new ToolPackageInstallerMock(
-                _fileSystem,
+
+            _toolPackageDownloader = new ToolPackageDownloader(toolPackageStoreMock);
+
+            _toolPackageDownloaderMock = new ToolPackageDownloaderMock(
                 _toolPackageStore,
-                new ProjectRestorerMock(
-                    _fileSystem,
-                    _reporter,
-                    new List<MockFeed>
+                _fileSystem,    
+                _reporter,
+                new List<MockFeed>
+                {
+                    new MockFeed
                     {
-                        new MockFeed
+                        Type = MockFeedType.ImplicitAdditionalFeed,
+                        Packages = new List<MockFeedPackage>
                         {
-                            Type = MockFeedType.ImplicitAdditionalFeed,
-                            Packages = new List<MockFeedPackage>
+                            new MockFeedPackage
                             {
-                                new MockFeedPackage
-                                {
-                                    PackageId = _packageIdA.ToString(),
-                                    Version = _packageVersionA.ToNormalizedString(),
-                                    ToolCommandName = _toolCommandNameA.ToString()
-                                },
-                                new MockFeedPackage
-                                {
-                                    PackageId = _packageIdB.ToString(),
-                                    Version = _packageVersionB.ToNormalizedString(),
-                                    ToolCommandName = _toolCommandNameB.ToString()
-                                },
-                                new MockFeedPackage
-                                {
-                                    PackageId = _packageIdWithCommandNameCollisionWithA.ToString(),
-                                    Version = _packageVersionWithCommandNameCollisionWithA.ToNormalizedString(),
-                                    ToolCommandName = "A"
-                                }
+                                PackageId = _packageIdA.ToString(),
+                                Version = _packageVersionA.ToNormalizedString(),
+                                ToolCommandName = _toolCommandNameA.ToString()
+                            },
+                            new MockFeedPackage
+                            {
+                                PackageId = _packageIdB.ToString(),
+                                Version = _packageVersionB.ToNormalizedString(),
+                                ToolCommandName = _toolCommandNameB.ToString()
+                            },
+                            new MockFeedPackage
+                            {
+                                PackageId = _packageIdWithCommandNameCollisionWithA.ToString(),
+                                Version = _packageVersionWithCommandNameCollisionWithA.ToNormalizedString(),
+                                ToolCommandName = "A"
                             }
                         }
-                    }),
-                installCallback: () => _installCalledCount++);
+                    }
+                },
+                downloadCallback: () => _installCalledCount++);
 
             _parseResult = Parser.Instance.Parse("dotnet tool restore");
 
@@ -123,7 +126,7 @@ namespace Microsoft.DotNet.Tests.Commands.Tool
                 });
 
             ToolRestoreCommand toolRestoreCommand = new ToolRestoreCommand(_parseResult,
-                _toolPackageInstallerMock,
+                _toolPackageDownloaderMock,
                 manifestFinder,
                 _localToolsResolverCache,
                 _fileSystem,
@@ -160,7 +163,7 @@ namespace Microsoft.DotNet.Tests.Commands.Tool
                 });
 
             ToolRestoreCommand toolRestoreCommand = new ToolRestoreCommand(_parseResult,
-                _toolPackageInstallerMock,
+                _toolPackageDownloaderMock,
                 manifestFinder,
                 _localToolsResolverCache,
                 _fileSystem,
@@ -195,7 +198,7 @@ namespace Microsoft.DotNet.Tests.Commands.Tool
                 });
 
             ToolRestoreCommand toolRestoreCommand = new ToolRestoreCommand(_parseResult,
-                _toolPackageInstallerMock,
+                _toolPackageDownloaderMock,
                 manifestFinder,
                 _localToolsResolverCache,
                 _fileSystem,
@@ -230,7 +233,7 @@ namespace Microsoft.DotNet.Tests.Commands.Tool
             };
 
             Action a = () => toolRestoreCommand.Execute();
-            a.ShouldThrow<ToolPackageException>()
+            a.Should().Throw<ToolPackageException>()
                 .And.Message
                 .Should().BeOneOf(allPossibleErrorMessage, "Run in parallel, no order guarantee");
         }
@@ -250,7 +253,7 @@ namespace Microsoft.DotNet.Tests.Commands.Tool
                 });
 
             ToolRestoreCommand toolRestoreCommand = new ToolRestoreCommand(_parseResult,
-                _toolPackageInstallerMock,
+                _toolPackageDownloaderMock,
                 manifestFinder,
                 _localToolsResolverCache,
                 _fileSystem,
@@ -290,7 +293,7 @@ namespace Microsoft.DotNet.Tests.Commands.Tool
                 });
 
             ToolRestoreCommand toolRestoreCommand = new ToolRestoreCommand(_parseResult,
-                _toolPackageInstallerMock,
+                _toolPackageDownloaderMock,
                 manifestFinder,
                 _localToolsResolverCache,
                 _fileSystem,
@@ -305,6 +308,126 @@ namespace Microsoft.DotNet.Tests.Commands.Tool
                             "\"different-command-nameA\" \"different-command-nameB\"", _packageIdA, "\"a\"")));
         }
 
+        [Fact]
+        public void ItRestoresMultipleTools()
+        {
+            var testDir = _testAssetsManager.CreateTestDirectory().Path;
+
+            string configContents = """
+                {
+                  "version": 1,
+                  "isRoot": true,
+                  "tools": {
+                    "cake.tool": {
+                      "version": "2.3.0",
+                      "commands": [
+                        "dotnet-cake"
+                      ]
+                    },
+                    "powershell": {
+                      "version": "7.3.7",
+                      "commands": [
+                        "pwsh"
+                      ]
+                    },
+                    "api-tools": {
+                      "version": "1.3.5",
+                      "commands": [
+                        "api-tools"
+                      ]
+                    },
+                    "dotnet-ef": {
+                      "version": "8.0.0-rc.1.23419.6",
+                      "commands": [
+                        "dotnet-ef"
+                      ]
+                    }
+                  }
+                }
+                """;
+
+            File.WriteAllText(Path.Combine(testDir, "dotnet-tools.json"), configContents);
+
+            string CliHome = Path.Combine(testDir, ".home");
+            Directory.CreateDirectory(CliHome);
+
+            var toolRestoreCommand = new DotnetCommand(Log, "tool", "restore")
+                .WithEnvironmentVariable("DOTNET_CLI_HOME", CliHome)
+                .WithEnvironmentVariable("DOTNET_SKIP_WORKLOAD_INTEGRITY_CHECK", "true")
+                .WithWorkingDirectory(testDir);
+
+            toolRestoreCommand
+                .Execute()
+                .Should()
+                .Pass();
+
+            //  Delete tool resolver cache and then run command again.  NuGet packages will still be downloaded to packages folder, making it more likely to hit concurrency issues
+            //  in the tool code
+            Directory.Delete(CliHome, true);
+
+            toolRestoreCommand
+                .Execute()
+                .Should()
+                .Pass();
+        }
+
+        private class CacheRow
+        {
+            public string Version { get; set; }
+            public string TargetFramework { get; set; }
+            public string RuntimeIdentifier { get; set; }
+            public string Name { get; set; }
+            public string Runner { get; set; }
+            public string PathToExecutable { get; set; }
+        }
+
+        [Fact]
+        public void ItRestoresCorrectToolVersion()
+        {
+            var testDir = _testAssetsManager.CreateTestDirectory().Path;
+
+            string configContents = """
+                {
+                  "version": 1,
+                  "isRoot": true,
+                  "tools": {
+                    "dotnet-ef": {
+                      "version": "8.0.0-rc.1.23419.6",
+                      "commands": [
+                        "dotnet-ef"
+                      ]
+                    }
+                  }
+                }
+                """;
+
+            File.WriteAllText(Path.Combine(testDir, "dotnet-tools.json"), configContents);
+
+            string CliHome = Path.Combine(testDir, ".home");
+            Directory.CreateDirectory(CliHome);
+
+            var toolRestoreCommand = new DotnetCommand(Log, "tool", "restore")
+                .WithEnvironmentVariable("DOTNET_CLI_HOME", CliHome)
+                .WithEnvironmentVariable("DOTNET_SKIP_WORKLOAD_INTEGRITY_CHECK", "true")
+                .WithWorkingDirectory(testDir);
+
+            toolRestoreCommand
+                .Execute()
+                .Should()
+                .Pass();
+
+            var cacheFilePath = Path.Combine(CliHome, ".dotnet", "toolResolverCache", "1", "dotnet-ef");
+
+            string json = File.ReadAllText(cacheFilePath);
+
+            var rows = JsonSerializer.Deserialize<List<CacheRow>>(json);
+
+            rows.Count.Should().Be(1);
+
+            rows[0].Name.Should().Be("dotnet-ef");
+            rows[0].Version.Should().Be("8.0.0-rc.1.23419.6");
+        }
+
         [Fact]
         public void WhenCannotFindManifestFileItPrintsWarning()
         {
@@ -312,7 +435,7 @@ namespace Microsoft.DotNet.Tests.Commands.Tool
                 new ToolManifestFinder(new DirectoryPath(Path.GetTempPath()), _fileSystem, new FakeDangerousFileDetector());
 
             ToolRestoreCommand toolRestoreCommand = new ToolRestoreCommand(_parseResult,
-                _toolPackageInstallerMock,
+                _toolPackageDownloaderMock,
                 realManifestFinderImplementationWithMockFinderSystem,
                 _localToolsResolverCache,
                 _fileSystem,
@@ -338,7 +461,7 @@ namespace Microsoft.DotNet.Tests.Commands.Tool
                 });
 
             ToolRestoreCommand toolRestoreCommand = new ToolRestoreCommand(_parseResult,
-                _toolPackageInstallerMock,
+                _toolPackageDownloaderMock,
                 manifestFinder,
                 _localToolsResolverCache,
                 _fileSystem,
@@ -365,7 +488,7 @@ namespace Microsoft.DotNet.Tests.Commands.Tool
                 });
 
             ToolRestoreCommand toolRestoreCommand = new ToolRestoreCommand(_parseResult,
-                _toolPackageInstallerMock,
+                _toolPackageDownloaderMock,
                 manifestFinder,
                 _localToolsResolverCache,
                 _fileSystem,
@@ -388,7 +511,7 @@ namespace Microsoft.DotNet.Tests.Commands.Tool
                 new CannotFindManifestFinder();
 
             ToolRestoreCommand toolRestoreCommand = new ToolRestoreCommand(_parseResult,
-                _toolPackageInstallerMock,
+                _toolPackageDownloaderMock,
                 manifestFinder,
                 _localToolsResolverCache,
                 _fileSystem,
@@ -398,7 +521,7 @@ namespace Microsoft.DotNet.Tests.Commands.Tool
             toolRestoreCommand.Execute().Should().Be(0);
 
             _reporter.Lines.Should().Contain(l =>
-                l.Contains(Cli.Utils.AnsiColorExtensions.Yellow(LocalizableStrings.NoToolsWereRestored)));
+                l.Contains(Cli.Utils.AnsiExtensions.Yellow(LocalizableStrings.NoToolsWereRestored)));
         }
 
         private class MockManifestFinder : IToolManifestFinder
@@ -415,7 +538,7 @@ namespace Microsoft.DotNet.Tests.Commands.Tool
                 return _toReturn;
             }
 
-            public FilePath FindFirst()
+            public FilePath FindFirst(bool createManifestFileOption = false)
             {
                 throw new NotImplementedException();
             }
@@ -433,7 +556,7 @@ namespace Microsoft.DotNet.Tests.Commands.Tool
                 throw new ToolManifestCannotBeFoundException("In test cannot find manifest");
             }
 
-            public FilePath FindFirst()
+            public FilePath FindFirst(bool createManifestFileOption = false)
             {
                 throw new NotImplementedException();
             }
@@ -445,3 +568,4 @@ namespace Microsoft.DotNet.Tests.Commands.Tool
         }
     }
 }
+
