@@ -1,11 +1,6 @@
-// Copyright (c) .NET Foundation and contributors. All rights reserved.
-// Licensed under the MIT license. See LICENSE file in the project root for full license information.
+// Licensed to the .NET Foundation under one or more agreements.
+// The .NET Foundation licenses this file to you under the MIT license.
 
-using System;
-using System.IO;
-using System.Linq;
-using System.Runtime.InteropServices;
-using FluentAssertions;
 using Microsoft.DotNet.Cli.Utils;
 using Microsoft.DotNet.ToolPackage;
 using Microsoft.DotNet.Tools;
@@ -14,12 +9,10 @@ using Microsoft.DotNet.Tools.Tool.Uninstall;
 using Microsoft.DotNet.Tools.Tests.ComponentMocks;
 using Microsoft.Extensions.DependencyModel.Tests;
 using Microsoft.Extensions.EnvironmentAbstractions;
-using Xunit;
 using LocalizableStrings = Microsoft.DotNet.Tools.Tool.Uninstall.LocalizableStrings;
 using InstallLocalizableStrings = Microsoft.DotNet.Tools.Tool.Install.LocalizableStrings;
 using Microsoft.DotNet.ShellShim;
-using Microsoft.NET.TestFramework.Utilities;
-using System.CommandLine.Parsing;
+using System.CommandLine;
 using Parser = Microsoft.DotNet.Cli.Parser;
 
 namespace Microsoft.DotNet.Tests.Commands.Tool
@@ -53,7 +46,7 @@ namespace Microsoft.DotNet.Tests.Commands.Tool
 
             Action a = () => command.Execute();
 
-            a.ShouldThrow<GracefulException>()
+            a.Should().Throw<GracefulException>()
                 .And
                 .Message
                 .Should()
@@ -101,7 +94,7 @@ namespace Microsoft.DotNet.Tests.Commands.Tool
             _fileSystem.Directory.Exists(packageDirectory.Value).Should().BeFalse();
             _fileSystem.File.Exists(shimPath).Should().BeFalse();
         }
-        
+
         [Fact]
         public void GivenAPackageWhenCallFromUninstallRedirectCommandItUninstalls()
         {
@@ -129,7 +122,7 @@ namespace Microsoft.DotNet.Tests.Commands.Tool
 
             _reporter.Lines.Clear();
 
-            
+
             ParseResult result = Parser.Instance.Parse("dotnet tool uninstall " + $"-g {PackageId}");
 
             (IToolPackageStore, IToolPackageStoreQuery, IToolPackageUninstaller) CreateToolPackageStoreAndUninstaller(
@@ -151,14 +144,14 @@ namespace Microsoft.DotNet.Tests.Commands.Tool
                     fileSystem: _fileSystem,
                     appHostShellShimMaker: new AppHostShellShimMakerMock(_fileSystem)),
                 _reporter);
-            
-            var uninstallCommand 
+
+            var uninstallCommand
                 = new ToolUninstallCommand(
-                    result, 
-                    toolUninstallGlobalOrToolPathCommand: toolUninstallGlobalOrToolPathCommand) ;
+                    result,
+                    toolUninstallGlobalOrToolPathCommand: toolUninstallGlobalOrToolPathCommand);
 
             uninstallCommand.Execute().Should().Be(0);
-            
+
             _reporter
                 .Lines
                 .Single()
@@ -202,7 +195,7 @@ namespace Microsoft.DotNet.Tests.Commands.Tool
                 uninstallCallback: () => throw new IOException("simulated error"))
                 .Execute();
 
-            a.ShouldThrow<GracefulException>()
+            a.Should().Throw<GracefulException>()
                 .And
                 .Message
                 .Should()
@@ -224,7 +217,7 @@ namespace Microsoft.DotNet.Tests.Commands.Tool
 
             Action a = () => uninstallCommand.Execute();
 
-            a.ShouldThrow<GracefulException>()
+            a.Should().Throw<GracefulException>()
                 .And
                 .Message
                 .Should()
@@ -236,16 +229,16 @@ namespace Microsoft.DotNet.Tests.Commands.Tool
             ParseResult result = Parser.Instance.Parse("dotnet tool install " + options);
 
             var store = new ToolPackageStoreMock(new DirectoryPath(_toolsDirectory), _fileSystem);
-            var packageInstallerMock = new ToolPackageInstallerMock(
-                _fileSystem,
-                store,
-                new ProjectRestorerMock(
-                    _fileSystem,
-                    _reporter));
+
+            var packageDownloaderMock = new ToolPackageDownloaderMock(
+                    store: store,
+                    fileSystem: _fileSystem,
+                    _reporter
+                    );
 
             return new ToolInstallGlobalOrToolPathCommand(
                 result,
-                (location, forwardArguments) => (store, store, packageInstallerMock),
+                (location, forwardArguments) => (store, store, packageDownloaderMock),
                 (_, _) => new ShellShimRepository(
                     new DirectoryPath(_shimsDirectory),
                     string.Empty,
@@ -281,3 +274,4 @@ namespace Microsoft.DotNet.Tests.Commands.Tool
         }
     }
 }
+
