@@ -8,7 +8,6 @@
 [assembly: System.Runtime.CompilerServices.RuntimeCompatibility(WrapNonExceptionThrows = true)]
 [assembly: System.Diagnostics.Debuggable(System.Diagnostics.DebuggableAttribute.DebuggingModes.IgnoreSymbolStoreSequencePoints)]
 [assembly: System.Runtime.CompilerServices.InternalsVisibleTo("System.Collections.Immutable.Tests, PublicKey=002400000480000094000000060200000024000052534131000400000100010015c01ae1f50e8cc09ba9eac9147cf8fd9fce2cfe9f8dce4f7301c4132ca9fb50ce8cbf1df4dc18dd4d210e4345c744ecb3365ed327efdbc52603faa5e21daa11234c8c4a73e51f03bf192544581ebe107adee3a34928e39d04e524a9ce729d5090bfd7dad9d10c722c0def9ccc08ff0a03790e48bcd1f9b6c476063e1966a1c4")]
-[assembly: System.Security.SecurityTransparent]
 [assembly: System.Resources.NeutralResourcesLanguage("en-US")]
 [assembly: System.Reflection.AssemblyTitle("System.Collections.Immutable")]
 [assembly: System.Reflection.AssemblyDescription("System.Collections.Immutable")]
@@ -16,17 +15,18 @@
 [assembly: System.Reflection.AssemblyCompany("Microsoft Corporation")]
 [assembly: System.Reflection.AssemblyProduct("Microsoft® .NET Framework")]
 [assembly: System.Reflection.AssemblyCopyright("© Microsoft Corporation.  All rights reserved.")]
-[assembly: System.Reflection.AssemblyFileVersion("4.6.24705.01")]
-[assembly: System.Reflection.AssemblyInformationalVersion("4.6.24705.01. Commit Hash: 4d1af962ca0fede10beb01d197367c2f90e92c97")]
+[assembly: System.Reflection.AssemblyFileVersion("4.6.26515.06")]
+[assembly: System.Reflection.AssemblyInformationalVersion("4.6.26515.06 @BuiltBy: dlab-DDVSOWINAGE059 @Branch: release/2.1 @SrcCode: https://github.com/dotnet/corefx/tree/30ab651fcb4354552bd4891619a0bdd81e0ebdbf")]
 [assembly: System.CLSCompliant(true)]
 [assembly: System.Reflection.AssemblyMetadata(".NETFrameworkAssembly", "")]
 [assembly: System.Reflection.AssemblyMetadata("Serviceable", "True")]
-[assembly: System.Reflection.AssemblyVersionAttribute("1.2.1.0")]
+[assembly: System.Reflection.AssemblyMetadata("PreferInbox", "True")]
+[assembly: System.Reflection.AssemblyVersionAttribute("1.2.3.0")]
 [assembly: System.Runtime.CompilerServices.ReferenceAssembly]
 [assembly: System.Reflection.AssemblyFlagsAttribute((System.Reflection.AssemblyNameFlags)0x70)]
 namespace System.Collections.Immutable
 {
-    public partial interface IImmutableDictionary<TKey, TValue> : Generic.IReadOnlyDictionary<TKey, TValue>, Generic.IReadOnlyCollection<Generic.KeyValuePair<TKey, TValue>>, Generic.IEnumerable<Generic.KeyValuePair<TKey, TValue>>, IEnumerable
+    public partial interface IImmutableDictionary<TKey, TValue> : Generic.IReadOnlyDictionary<TKey, TValue>, Generic.IEnumerable<Generic.KeyValuePair<TKey, TValue>>, IEnumerable, Generic.IReadOnlyCollection<Generic.KeyValuePair<TKey, TValue>>
     {
         IImmutableDictionary<TKey, TValue> Add(TKey key, TValue value);
         IImmutableDictionary<TKey, TValue> AddRange(Generic.IEnumerable<Generic.KeyValuePair<TKey, TValue>> pairs);
@@ -39,7 +39,7 @@ namespace System.Collections.Immutable
         bool TryGetKey(TKey equalKey, out TKey actualKey);
     }
 
-    public partial interface IImmutableList<T> : Generic.IReadOnlyList<T>, Generic.IReadOnlyCollection<T>, Generic.IEnumerable<T>, IEnumerable
+    public partial interface IImmutableList<T> : Generic.IReadOnlyList<T>, Generic.IEnumerable<T>, IEnumerable, Generic.IReadOnlyCollection<T>
     {
         IImmutableList<T> Add(T value);
         IImmutableList<T> AddRange(Generic.IEnumerable<T> items);
@@ -139,7 +139,7 @@ namespace System.Collections.Immutable
         public static ImmutableArray<TSource> ToImmutableArray<TSource>(this Generic.IEnumerable<TSource> items) { throw null; }
     }
 
-    public partial struct ImmutableArray<T> : Generic.IReadOnlyList<T>, Generic.IReadOnlyCollection<T>, Generic.IEnumerable<T>, IEnumerable, Generic.IList<T>, Generic.ICollection<T>, IEquatable<ImmutableArray<T>>, IImmutableList<T>, IList, ICollection, IStructuralComparable, IStructuralEquatable
+    public partial struct ImmutableArray<T> : Generic.IReadOnlyList<T>, Generic.IEnumerable<T>, IEnumerable, Generic.IReadOnlyCollection<T>, Generic.IList<T>, Generic.ICollection<T>, IEquatable<ImmutableArray<T>>, IList, ICollection, IStructuralComparable, IStructuralEquatable, IImmutableList<T>
     {
         private object _dummy;
         private int _dummyPrimitive;
@@ -225,6 +225,8 @@ namespace System.Collections.Immutable
 
         public ImmutableArray<T> InsertRange(int index, ImmutableArray<T> items) { throw null; }
 
+        public ref readonly T ItemRef(int index) { throw null; }
+
         public int LastIndexOf(T item, int startIndex, int count, Generic.IEqualityComparer<T> equalityComparer) { throw null; }
 
         public int LastIndexOf(T item, int startIndex, int count) { throw null; }
@@ -390,6 +392,8 @@ namespace System.Collections.Immutable
 
             public void Insert(int index, T item) { }
 
+            public ref readonly T ItemRef(int index) { throw null; }
+
             public int LastIndexOf(T item, int startIndex, int count, Generic.IEqualityComparer<T> equalityComparer) { throw null; }
 
             public int LastIndexOf(T item, int startIndex, int count) { throw null; }
@@ -476,7 +480,7 @@ namespace System.Collections.Immutable
         public static ImmutableDictionary<TKey, TValue> ToImmutableDictionary<TSource, TKey, TValue>(this Generic.IEnumerable<TSource> source, Func<TSource, TKey> keySelector, Func<TSource, TValue> elementSelector) { throw null; }
     }
 
-    public sealed partial class ImmutableDictionary<TKey, TValue> : IImmutableDictionary<TKey, TValue>, Generic.IReadOnlyDictionary<TKey, TValue>, Generic.IReadOnlyCollection<Generic.KeyValuePair<TKey, TValue>>, Generic.IEnumerable<Generic.KeyValuePair<TKey, TValue>>, IEnumerable, Generic.IDictionary<TKey, TValue>, Generic.ICollection<Generic.KeyValuePair<TKey, TValue>>, IDictionary, ICollection
+    public sealed partial class ImmutableDictionary<TKey, TValue> : IImmutableDictionary<TKey, TValue>, Generic.IReadOnlyDictionary<TKey, TValue>, Generic.IEnumerable<Generic.KeyValuePair<TKey, TValue>>, IEnumerable, Generic.IReadOnlyCollection<Generic.KeyValuePair<TKey, TValue>>, Generic.IDictionary<TKey, TValue>, Generic.ICollection<Generic.KeyValuePair<TKey, TValue>>, IDictionary, ICollection
     {
         internal ImmutableDictionary() { }
 
@@ -953,7 +957,7 @@ namespace System.Collections.Immutable
         public static ImmutableList<TSource> ToImmutableList<TSource>(this Generic.IEnumerable<TSource> source) { throw null; }
     }
 
-    public sealed partial class ImmutableList<T> : IImmutableList<T>, Generic.IReadOnlyList<T>, Generic.IReadOnlyCollection<T>, Generic.IEnumerable<T>, IEnumerable, Generic.IList<T>, Generic.ICollection<T>, IList, ICollection
+    public sealed partial class ImmutableList<T> : IImmutableList<T>, Generic.IReadOnlyList<T>, Generic.IEnumerable<T>, IEnumerable, Generic.IReadOnlyCollection<T>, Generic.IList<T>, Generic.ICollection<T>, IList, ICollection
     {
         internal ImmutableList() { }
 
@@ -1034,6 +1038,8 @@ namespace System.Collections.Immutable
 
         public ImmutableList<T> InsertRange(int index, Generic.IEnumerable<T> items) { throw null; }
 
+        public ref readonly T ItemRef(int index) { throw null; }
+
         public int LastIndexOf(T item, int index, int count, Generic.IEqualityComparer<T> equalityComparer) { throw null; }
 
         public ImmutableList<T> Remove(T value, Generic.IEqualityComparer<T> equalityComparer) { throw null; }
@@ -1206,6 +1212,8 @@ namespace System.Collections.Immutable
 
             public void InsertRange(int index, Generic.IEnumerable<T> items) { }
 
+            public ref readonly T ItemRef(int index) { throw null; }
+
             public int LastIndexOf(T item, int startIndex, int count, Generic.IEqualityComparer<T> equalityComparer) { throw null; }
 
             public int LastIndexOf(T item, int startIndex, int count) { throw null; }
@@ -1304,6 +1312,8 @@ namespace System.Collections.Immutable
 
         public T Peek() { throw null; }
 
+        public ref readonly T PeekRef() { throw null; }
+
         Generic.IEnumerator<T> Generic.IEnumerable<T>.GetEnumerator() { throw null; }
 
         IEnumerator IEnumerable.GetEnumerator() { throw null; }
@@ -1360,7 +1370,7 @@ namespace System.Collections.Immutable
         public static ImmutableSortedDictionary<TKey, TValue> ToImmutableSortedDictionary<TSource, TKey, TValue>(this Generic.IEnumerable<TSource> source, Func<TSource, TKey> keySelector, Func<TSource, TValue> elementSelector) { throw null; }
     }
 
-    public sealed partial class ImmutableSortedDictionary<TKey, TValue> : IImmutableDictionary<TKey, TValue>, Generic.IReadOnlyDictionary<TKey, TValue>, Generic.IReadOnlyCollection<Generic.KeyValuePair<TKey, TValue>>, Generic.IEnumerable<Generic.KeyValuePair<TKey, TValue>>, IEnumerable, Generic.IDictionary<TKey, TValue>, Generic.ICollection<Generic.KeyValuePair<TKey, TValue>>, IDictionary, ICollection
+    public sealed partial class ImmutableSortedDictionary<TKey, TValue> : IImmutableDictionary<TKey, TValue>, Generic.IReadOnlyDictionary<TKey, TValue>, Generic.IEnumerable<Generic.KeyValuePair<TKey, TValue>>, IEnumerable, Generic.IReadOnlyCollection<Generic.KeyValuePair<TKey, TValue>>, Generic.IDictionary<TKey, TValue>, Generic.ICollection<Generic.KeyValuePair<TKey, TValue>>, IDictionary, ICollection
     {
         internal ImmutableSortedDictionary() { }
 
@@ -1471,6 +1481,8 @@ namespace System.Collections.Immutable
 
         public bool TryGetValue(TKey key, out TValue value) { throw null; }
 
+        public ref readonly TValue ValueRef(TKey key) { throw null; }
+
         public ImmutableSortedDictionary<TKey, TValue> WithComparers(Generic.IComparer<TKey> keyComparer, Generic.IEqualityComparer<TValue> valueComparer) { throw null; }
 
         public ImmutableSortedDictionary<TKey, TValue> WithComparers(Generic.IComparer<TKey> keyComparer) { throw null; }
@@ -1558,6 +1570,8 @@ namespace System.Collections.Immutable
             public bool TryGetKey(TKey equalKey, out TKey actualKey) { throw null; }
 
             public bool TryGetValue(TKey key, out TValue value) { throw null; }
+
+            public ref readonly TValue ValueRef(TKey key) { throw null; }
         }
 
         public partial struct Enumerator : Generic.IEnumerator<Generic.KeyValuePair<TKey, TValue>>, IEnumerator, IDisposable
@@ -1656,6 +1670,8 @@ namespace System.Collections.Immutable
 
         public bool IsSupersetOf(Generic.IEnumerable<T> other) { throw null; }
 
+        public ref readonly T ItemRef(int index) { throw null; }
+
         public bool Overlaps(Generic.IEnumerable<T> other) { throw null; }
 
         public ImmutableSortedSet<T> Remove(T value) { throw null; }
@@ -1770,6 +1786,8 @@ namespace System.Collections.Immutable
 
             public bool IsSupersetOf(Generic.IEnumerable<T> other) { throw null; }
 
+            public ref readonly T ItemRef(int index) { throw null; }
+
             public bool Overlaps(Generic.IEnumerable<T> other) { throw null; }
 
             public bool Remove(T item) { throw null; }
@@ -1838,6 +1856,8 @@ namespace System.Collections.Immutable
 
         public T Peek() { throw null; }
 
+        public ref readonly T PeekRef() { throw null; }
+
         public ImmutableStack<T> Pop() { throw null; }
 
         public ImmutableStack<T> Pop(out T value) { throw null; }
