@@ -7,25 +7,25 @@
 [assembly: System.Runtime.CompilerServices.CompilationRelaxations(8)]
 [assembly: System.Runtime.CompilerServices.RuntimeCompatibility(WrapNonExceptionThrows = true)]
 [assembly: System.Diagnostics.Debuggable(System.Diagnostics.DebuggableAttribute.DebuggingModes.IgnoreSymbolStoreSequencePoints)]
-[assembly: System.Runtime.Versioning.TargetFramework(".NETCoreApp,Version=v6.0", FrameworkDisplayName = ".NET 6.0")]
+[assembly: System.Runtime.Versioning.TargetFramework(".NETCoreApp,Version=v6.0", FrameworkDisplayName = "")]
+[assembly: System.CLSCompliant(true)]
+[assembly: System.Runtime.InteropServices.DefaultDllImportSearchPaths(System.Runtime.InteropServices.DllImportSearchPath.AssemblyDirectory | System.Runtime.InteropServices.DllImportSearchPath.System32)]
+[assembly: System.Runtime.Versioning.UnsupportedOSPlatform("browser")]
+[assembly: System.Reflection.AssemblyDefaultAlias("System.Security.Cryptography.Pkcs")]
+[assembly: System.Resources.NeutralResourcesLanguage("en-US")]
 [assembly: System.Reflection.AssemblyMetadata(".NETFrameworkAssembly", "")]
 [assembly: System.Reflection.AssemblyMetadata("Serviceable", "True")]
 [assembly: System.Reflection.AssemblyMetadata("PreferInbox", "True")]
-[assembly: System.Reflection.AssemblyDefaultAlias("System.Security.Cryptography.Pkcs")]
-[assembly: System.Resources.NeutralResourcesLanguage("en-US")]
-[assembly: System.CLSCompliant(true)]
 [assembly: System.Reflection.AssemblyMetadata("IsTrimmable", "True")]
-[assembly: System.Runtime.Versioning.UnsupportedOSPlatform("browser")]
-[assembly: System.Runtime.InteropServices.DefaultDllImportSearchPaths(System.Runtime.InteropServices.DllImportSearchPath.AssemblyDirectory | System.Runtime.InteropServices.DllImportSearchPath.System32)]
 [assembly: System.Reflection.AssemblyCompany("Microsoft Corporation")]
 [assembly: System.Reflection.AssemblyCopyright("© Microsoft Corporation. All rights reserved.")]
 [assembly: System.Reflection.AssemblyDescription("Provides support for PKCS and CMS algorithms.\r\n\r\nCommonly Used Types:\r\nSystem.Security.Cryptography.Pkcs.EnvelopedCms")]
-[assembly: System.Reflection.AssemblyFileVersion("7.0.723.27404")]
-[assembly: System.Reflection.AssemblyInformationalVersion("7.0.7+5b20af47d99620150c53eaf5db8636fdf730b126")]
+[assembly: System.Reflection.AssemblyFileVersion("6.0.522.21309")]
+[assembly: System.Reflection.AssemblyInformationalVersion("6.0.5+70ae3df4a6f3c92fb6b315afc405edd10ff38579")]
 [assembly: System.Reflection.AssemblyProduct("Microsoft® .NET")]
 [assembly: System.Reflection.AssemblyTitle("System.Security.Cryptography.Pkcs")]
 [assembly: System.Reflection.AssemblyMetadata("RepositoryUrl", "https://github.com/dotnet/runtime")]
-[assembly: System.Reflection.AssemblyVersionAttribute("7.0.0.0")]
+[assembly: System.Reflection.AssemblyVersionAttribute("6.0.0.0")]
 [assembly: System.Runtime.CompilerServices.ReferenceAssembly]
 [assembly: System.Reflection.AssemblyFlagsAttribute((System.Reflection.AssemblyNameFlags)0x70)]
 namespace System.Security.Cryptography
@@ -169,8 +169,6 @@ namespace System.Security.Cryptography.Pkcs
 
         public CmsSigner(SubjectIdentifierType signerIdentifierType, X509Certificates.X509Certificate2? certificate, AsymmetricAlgorithm? privateKey) { }
 
-        public CmsSigner(SubjectIdentifierType signerIdentifierType, X509Certificates.X509Certificate2? certificate, RSA? privateKey, RSASignaturePadding? signaturePadding) { }
-
         public CmsSigner(SubjectIdentifierType signerIdentifierType, X509Certificates.X509Certificate2? certificate) { }
 
         public CmsSigner(SubjectIdentifierType signerIdentifierType) { }
@@ -187,8 +185,6 @@ namespace System.Security.Cryptography.Pkcs
 
         public AsymmetricAlgorithm? PrivateKey { get { throw null; } set { } }
 
-        public RSASignaturePadding? SignaturePadding { get { throw null; } set { } }
-
         public CryptographicAttributeObjectCollection SignedAttributes { get { throw null; } }
 
         public SubjectIdentifierType SignerIdentifierType { get { throw null; } set { } }
