@@ -1,12 +1,11 @@
 // Copyright (c) Microsoft Corporation. All rights reserved.
 // Licensed under the MIT license. See LICENSE file in the project root for full license information.
 
-namespace Microsoft.VisualStudio.TestPlatform.Common.Utilities
+namespace Microsoft.VisualStudio.TestPlatform.Common.Utilities;
+
+public enum AssemblyType
 {
-    public enum AssemblyType
-    {
-        None,
-        Native,
-        Managed
-    }
+    None,
+    Native,
+    Managed
 }
